!!
!! This module computes hydrologic processes on continental points.
!!
!! @author Marie-Alice Foujols and Jan Polcher
!! @Version : $Revision: 1.24.2.1 $, $Date: 2003/11/18 16:23:30 $
!! 
!! $Header: /home/ssipsl/CVSREP/ORCHIDEE/src_sechiba/hydrol.f90,v 1.24.2.1 2003/11/18 16:23:30 ssipsl Exp $
!! 
MODULE hydrol
  !
  !
  ! routines called : restput, restget
  !
  USE ioipsl
  !
  ! modules used :
  USE constantes
  USE constantes_veg
  USE sechiba_io

  IMPLICIT NONE

  ! public routines :
  ! hydrol
  PRIVATE
  PUBLIC :: hydrol_main,hydrol_clear 

  !
  ! variables used inside hydrol module : declaration and initialisation
  !
  LOGICAL, SAVE                                     :: l_first_hydrol=.TRUE. !! Initialisation has to be done one time
  !
  LOGICAL, SAVE                                     :: check_waterbal=.FALSE. !! The check the water balance
  LOGICAL, SAVE                                     :: ok_hdiff         !! do horizontal diffusion?

  CHARACTER*80 , SAVE                               :: file_ext         !! Extention for I/O filename
  CHARACTER*80 , SAVE                               :: var_name         !! To store variables names for I/O

  ! one dimension array allocated, computed, saved and got in hydrol module

  REAL(stnd), ALLOCATABLE, SAVE, DIMENSION (:,:)    :: bqsb             !! Hauteur d'eau dans le reservoir profond
  REAL(stnd), ALLOCATABLE, SAVE, DIMENSION (:,:)    :: gqsb             !! Hauteur d'eau dans le reservoir de surface
  REAL(stnd), ALLOCATABLE, SAVE, DIMENSION (:,:)    :: dsg              !! Hauteur du reservoir de surface
  REAL(stnd), ALLOCATABLE, SAVE, DIMENSION (:,:)    :: dsp              !! Hauteur au dessus du reservoir profond 
  REAL(stnd), ALLOCATABLE, SAVE, DIMENSION (:)      :: mean_bqsb        !! diagnostique du reservoir profond
  REAL(stnd), ALLOCATABLE, SAVE, DIMENSION (:)      :: mean_gqsb        !! diagnostique du reservoir de surface
  REAL(stnd), ALLOCATABLE, SAVE, DIMENSION (:)      :: tot_water_beg    !! Total amount of water at start of time step
  REAL(stnd), ALLOCATABLE, SAVE, DIMENSION (:)      :: tot_water_end    !! Total amount of water at end of time step
  REAL(stnd), ALLOCATABLE, SAVE, DIMENSION (:)      :: tot_watveg_beg   !! Total amount of water on vegetation at start of time step
  REAL(stnd), ALLOCATABLE, SAVE, DIMENSION (:)      :: tot_watveg_end   !! Total amount of water on vegetation at end of time step
  REAL(stnd), ALLOCATABLE, SAVE, DIMENSION (:)      :: tot_watsoil_beg  !! Total amount of water in the soil at start of time step
  REAL(stnd), ALLOCATABLE, SAVE, DIMENSION (:)      :: tot_watsoil_end  !! Total amount of water in the soil at end of time step
  REAL(stnd), ALLOCATABLE, SAVE, DIMENSION (:)      :: snow_beg         !! Total amount of snow at start of time step
  REAL(stnd), ALLOCATABLE, SAVE, DIMENSION (:)      :: snow_end         !! Total amount of snow at end of time step
  REAL(stnd), ALLOCATABLE, SAVE, DIMENSION (:)      :: delsoilmoist     !! Change in soil moisture
  REAL(stnd), ALLOCATABLE, SAVE, DIMENSION (:)      :: delintercept     !! Change in interception storage 
  REAL(stnd), ALLOCATABLE, SAVE, DIMENSION (:)      :: delswe           !! Change in SWE

  ! one dimension array allocated, computed and used in hydrol module exclusively

  REAL(stnd), ALLOCATABLE, SAVE, DIMENSION (:,:)    :: dss              !! Hauteur au dessus du reservoir de surface
  REAL(stnd), ALLOCATABLE, SAVE, DIMENSION (:,:)    :: precisol         !! Eau tombee sur le sol
  REAL(stnd), ALLOCATABLE, SAVE, DIMENSION (:)      :: subsnowveg       !! Sublimation of snow on vegetation
  REAL(stnd), ALLOCATABLE, SAVE, DIMENSION (:,:)    :: subsnownobio     !! Sublimation of snow on other surface types (ice, lakes, ...)
  REAL(stnd), ALLOCATABLE, SAVE, DIMENSION (:)      :: snowmelt         !! Quantite de neige fondue
  REAL(stnd), ALLOCATABLE, SAVE, DIMENSION (:)      :: icemelt          !! Quantite de glace fondue
  REAL(stnd), ALLOCATABLE, SAVE, DIMENSION (:,:)    :: gdrainage        !! Drainage between reservoirs


  REAL(stnd), ALLOCATABLE, SAVE, DIMENSION (:)      :: vegtot           !! Total  vegetation
  ! The last vegetation map which was used to distribute the reservoirs
  REAL(stnd), ALLOCATABLE, SAVE, DIMENSION (:,:)    :: resdist          !! Distribution of reservoirs

  !! profondeur du reservoir contenant le maximum d'eau  
  REAL(stnd), ALLOCATABLE, SAVE, DIMENSION (:)      :: mx_eau_var   
  REAL(stnd), ALLOCATABLE, SAVE, DIMENSION (:)      :: ruu_ch           !! Quantite d'eau maximum
  REAL(stnd), ALLOCATABLE, SAVE, DIMENSION (:,:)    :: runoff           !! Ruissellement

CONTAINS

  !! 
  !! Main routine for *hydrol* module
  !! - called only one time for initialisation
  !! - called every time step
  !! - called one more time at last time step for writing _restart_ file
  !!
  !! Algorithm:
  !! - call hydrol_snow for snow process (including age of snow)
  !! - call hydrol_canop for canopy process
  !! - call hydrol_soil for bare soil process
  !!
  !! @call hydrol_snow
  !! @call hydrol_canop
  !! @call hydrol_soil
  !!
  SUBROUTINE hydrol_main (kjit, kjpindex, dtradia, ldrestart_read, ldrestart_write, index, indexveg, &
     & temp_sol_new, run_off_tot, drainage, frac_nobio, totfrac_nobio, vevapwet, veget, veget_max,&
     & qsintmax, qsintveg, vevapnu, vevapsno, snow, snow_age, snow_nobio, snow_nobio_age, tot_melt, transpir, &
     & precip_rain, precip_snow, returnflow, irrigation, humrel, vegstress, hdry, evapot, evapot_corr,&
     & shumdiag, litterhumdiag, soilcap, rest_id, hist_id)   

    ! interface description
    ! input scalar 
    INTEGER(long), INTENT(in)                         :: kjit             !! Time step number 
    INTEGER(long), INTENT(in)                         :: kjpindex         !! Domain size
    INTEGER(long),INTENT (in)                         :: rest_id,hist_id  !! _Restart_ file and _history_ file identifier
    REAL(stnd), INTENT (in)                           :: dtradia          !! Time step in seconds
    LOGICAL, INTENT(in)                               :: ldrestart_read   !! Logical for _restart_ file to read
    LOGICAL, INTENT(in)                               :: ldrestart_write  !! Logical for _restart_ file to write
    ! input fields
    INTEGER(long),DIMENSION (kjpindex), INTENT (in)     :: index            !! Indeces of the points on the map
    INTEGER(long),DIMENSION (kjpindex*nvm), INTENT (in) :: indexveg         !! Indeces of the points on the 3D map
    REAL(stnd),DIMENSION (kjpindex), INTENT (in)      :: precip_rain      !! Rain precipitation
    REAL(stnd),DIMENSION (kjpindex), INTENT (in)      :: precip_snow      !! Snow precipitation
    REAL(stnd),DIMENSION (kjpindex), INTENT (in)      :: returnflow       !! Routed water which comes back into the soil
    REAL(stnd),DIMENSION (kjpindex), INTENT (in)      :: irrigation        !! Water from irrigation returning to soil moisture
    REAL(stnd),DIMENSION (kjpindex), INTENT (in)      :: temp_sol_new     !! New soil temperature
    REAL(stnd),DIMENSION (kjpindex,nnobio), INTENT (in) :: frac_nobio     !! Fraction of ice, lakes, ...
    REAL(stnd),DIMENSION (kjpindex), INTENT (in)      :: totfrac_nobio    !! Total fraction of ice+lakes+...
    REAL(stnd),DIMENSION (kjpindex), INTENT (in)      :: soilcap          !! Soil capacity
    REAL(stnd),DIMENSION (kjpindex,nvm), INTENT (in)  :: vevapwet         !! Interception loss
    REAL(stnd),DIMENSION (kjpindex,nvm), INTENT (in)  :: veget            !! Fraction of vegetation type           
    REAL(stnd),DIMENSION (kjpindex,nvm), INTENT (in)  :: veget_max        !! Max. fraction of vegetation type (LAI -> infty)
    REAL(stnd),DIMENSION (kjpindex,nvm), INTENT (in)  :: qsintmax         !! Maximum water on vegetation for interception
    REAL(stnd),DIMENSION (kjpindex,nvm), INTENT (in)  :: transpir         !! Transpiration
    REAL(stnd),DIMENSION (kjpindex), INTENT (in)      :: evapot           !! Soil Potential Evaporation
    REAL(stnd),DIMENSION (kjpindex), INTENT (in)      :: evapot_corr !! Soil Potential Evaporation Correction
    ! modified fields
    REAL(stnd),DIMENSION (kjpindex), INTENT (inout)   :: vevapnu          !! Bare soil evaporation
    REAL(stnd),DIMENSION (kjpindex), INTENT (inout)   :: vevapsno         !! Snow evaporation
    REAL(stnd),DIMENSION (kjpindex), INTENT (inout)   :: snow             !! Snow mass [Kg/m^2]
    REAL(stnd),DIMENSION (kjpindex), INTENT (inout)   :: snow_age         !! Snow age
    REAL(stnd),DIMENSION (kjpindex,nnobio), INTENT (inout) :: snow_nobio     !! Water balance on ice, lakes, .. [Kg/m^2]
    REAL(stnd),DIMENSION (kjpindex,nnobio), INTENT (inout) :: snow_nobio_age !! Snow age on ice, lakes, ...
    !! We consider that any water on the ice is snow and we only peforme a water balance to have consistency.
    !! The water balance is limite to + or - 10^6 so that accumulation is not endless
    ! output fields
    REAL(stnd),DIMENSION (kjpindex), INTENT (out)     :: run_off_tot   !! Complete runoff
    REAL(stnd),DIMENSION (kjpindex), INTENT (out)     :: drainage      !! Drainage
    REAL(stnd),DIMENSION (kjpindex,nvm), INTENT (out) :: humrel        !! Relative humidity
    REAL(stnd),DIMENSION (kjpindex,nvm), INTENT (out) :: vegstress     !! Veg. moisture stress (only for vegetation growth)
    REAL(stnd),DIMENSION (kjpindex), INTENT (out)     :: hdry          !! Dry soil heigth in meters
    REAL(stnd),DIMENSION (kjpindex,nbdl), INTENT (out):: shumdiag      !! relative soil moisture
    REAL(stnd),DIMENSION (kjpindex), INTENT (out)     :: litterhumdiag !! litter humidity
    REAL(stnd),DIMENSION (kjpindex), INTENT (out)     :: tot_melt      !! Total melt    
    REAL(stnd),DIMENSION (kjpindex,nvm), INTENT (out) :: qsintveg      !! Water on vegetation due to interception

    !
    ! local declaration
    !
    REAL(stnd),DIMENSION (kjpindex)                   :: soilwet          !! A temporary diagnostic of soil wetness
    REAL(stnd),DIMENSION (kjpindex)                   :: snowdepth        !! Depth of snow layer
    !
    ! do initialisation
    !
    IF (l_first_hydrol) THEN

        IF (long_print) WRITE (numout,*) ' l_first_hydrol : call hydrol_init '

        CALL hydrol_init (kjit, ldrestart_read, kjpindex, index, rest_id, veget, humrel, vegstress, hdry,&
             & snow, snow_age, snow_nobio, snow_nobio_age, qsintveg) 

        CALL hydrol_var_init (kjpindex, veget, veget_max, hdry, mx_eau_var, ruu_ch, shumdiag, litterhumdiag)
        !
        ! If we check the water balance we first save the total amount of water
        !
        IF (check_waterbal) THEN
           CALL hydrol_waterbal(kjpindex, index, .TRUE., dtradia, veget, totfrac_nobio, qsintveg, snow, snow_nobio,&
                & precip_rain, precip_snow, returnflow, irrigation, tot_melt, vevapwet, transpir, vevapnu, vevapsno,&
                & run_off_tot, drainage)
        ENDIF
        !
        IF (almaoutput) THEN
           CALL hydrol_alma(kjpindex, index, .TRUE., qsintveg, snow, snow_nobio, soilwet)
        ENDIF

        RETURN

    ENDIF

    !
    ! prepares restart file for the next simulation
    !
    IF (ldrestart_write) THEN

        IF (long_print) WRITE (numout,*) ' we have to complete restart file with HYDROLOGIC variables '

        var_name= 'humrel'  
        CALL restput(rest_id, var_name, kjpindex,   nvm, 1, kjit,  humrel, 'scatter',  kjpindex, index)
        !
        var_name= 'vegstress'  
        CALL restput(rest_id, var_name, kjpindex,   nvm, 1, kjit,  vegstress, 'scatter',  kjpindex, index)
        !
        var_name= 'hdry'
        CALL restput(rest_id, var_name, kjpindex,   1, 1, kjit,  hdry, 'scatter',  kjpindex, index)
        !
        var_name= 'snow'    
        CALL restput(rest_id, var_name, kjpindex,   1, 1, kjit,  snow, 'scatter',  kjpindex, index)
        !
        var_name= 'snow_age'
        CALL restput(rest_id, var_name, kjpindex,   1, 1, kjit,  snow_age, 'scatter',  kjpindex, index)
        !
        var_name= 'snow_nobio'    
        CALL restput(rest_id, var_name, kjpindex,   nnobio, 1, kjit,  snow_nobio, 'scatter',  kjpindex, index)
        !
        var_name= 'snow_nobio_age'
        CALL restput(rest_id, var_name, kjpindex,   nnobio, 1, kjit,  snow_nobio_age, 'scatter',  kjpindex, index)
        !
        var_name= 'bqsb'    
        CALL restput(rest_id, var_name, kjpindex,   nvm, 1, kjit,  bqsb, 'scatter',  kjpindex, index)
        !
        var_name= 'gqsb'   
        CALL restput(rest_id, var_name, kjpindex,   nvm, 1, kjit,  gqsb, 'scatter',  kjpindex, index)
        !
        var_name= 'dsg'     
        CALL restput(rest_id, var_name, kjpindex,  nvm, 1, kjit,  dsg, 'scatter',  kjpindex, index)
        !
        var_name= 'dsp'    
        CALL restput(rest_id, var_name, kjpindex,   nvm, 1, kjit,  dsp, 'scatter',  kjpindex, index)
        !
        var_name= 'qsintveg'
        CALL restput(rest_id, var_name, kjpindex, nvm, 1, kjit,  qsintveg, 'scatter',  kjpindex, index)
        !
        var_name= 'resdist'
        CALL restput(rest_id, var_name, kjpindex, nvm, 1, kjit,  resdist, 'scatter',  kjpindex, index)
        RETURN
        !
    END IF

    !
    ! shared time step
    !
    IF (long_print) WRITE (numout,*) 'pas de temps = ',dtradia

    ! 
    ! computes snow
    !

    CALL hydrol_snow(kjpindex, dtradia, precip_rain, precip_snow, temp_sol_new, soilcap, &
         & frac_nobio, totfrac_nobio, vevapnu, vevapsno, snow, snow_age, snow_nobio, snow_nobio_age, &
         & tot_melt, snowdepth)
    !
    ! computes canopy
    !
    !
    CALL hydrol_vegupd(kjpindex, veget, ruu_ch, qsintveg, gqsb, bqsb, dsg, dss,dsp, resdist)
    !
    CALL hydrol_canop(kjpindex, precip_rain, vevapwet, veget, qsintmax, qsintveg, precisol)
    !
    ! computes hydro_soil
    !
    CALL hydrol_soil(kjpindex, vevapnu, precisol, returnflow, irrigation, tot_melt, mx_eau_var, veget, ruu_ch, transpir,&
         & gqsb, bqsb, dsg, dss, hdry, dsp, runoff, run_off_tot, drainage, humrel, vegstress, shumdiag, litterhumdiag)
    !
    ! computes horizontal diffusion between the water reservoirs
    !
    IF ( ok_hdiff ) THEN
      CALL hydrol_hdiff(kjpindex, dtradia, veget, ruu_ch, gqsb, bqsb, dsg, dss, dsp)
    ENDIF
    !
    ! If we check the water balance we end with the comparison of total water change and fluxes
    !
    IF (check_waterbal) THEN
       CALL hydrol_waterbal(kjpindex, index, .FALSE., dtradia, veget, totfrac_nobio, qsintveg, snow, snow_nobio,&
            & precip_rain, precip_snow, returnflow, irrigation, tot_melt, vevapwet, transpir, vevapnu, vevapsno,&
            & run_off_tot, drainage )
    ENDIF
    !
    ! If we use the ALMA standards
    !
    IF (almaoutput) THEN
       CALL hydrol_alma(kjpindex, index, .FALSE., qsintveg, snow, snow_nobio, soilwet)
    ENDIF

    !
    IF ( .NOT. almaoutput ) THEN
       CALL histwrite(hist_id, 'bqsb', kjit, mean_bqsb, kjpindex, index)
       CALL histwrite(hist_id, 'gqsb', kjit, mean_gqsb, kjpindex, index)
       CALL histwrite(hist_id, 'runoff', kjit, run_off_tot, kjpindex, index)
       CALL histwrite(hist_id, 'drainage', kjit, drainage, kjpindex, index)
       CALL histwrite(hist_id, 'precisol', kjit, precisol, kjpindex*nvm, indexveg)
       CALL histwrite(hist_id, 'rain', kjit, precip_rain, kjpindex, index)
       CALL histwrite(hist_id, 'snowf', kjit, precip_snow, kjpindex, index)
       CALL histwrite(hist_id, 'qsintmax', kjit, qsintmax, kjpindex*nvm, indexveg)
       CALL histwrite(hist_id, 'qsintveg', kjit, qsintveg, kjpindex*nvm, indexveg)
       CALL histwrite(hist_id, 'humrel',   kjit, humrel,   kjpindex*nvm, indexveg)
    ELSE
       CALL histwrite(hist_id, 'Snowf', kjit, precip_snow, kjpindex, index)
       CALL histwrite(hist_id, 'Rainf', kjit, precip_rain, kjpindex, index)
       CALL histwrite(hist_id, 'Qs', kjit, run_off_tot, kjpindex, index)
       CALL histwrite(hist_id, 'Qsb', kjit, drainage, kjpindex, index)
       CALL histwrite(hist_id, 'Qsm', kjit, tot_melt, kjpindex, index)
       CALL histwrite(hist_id, 'DelSoilMoist', kjit, delsoilmoist, kjpindex, index)
       CALL histwrite(hist_id, 'DelSWE', kjit, delswe, kjpindex, index)
       CALL histwrite(hist_id, 'DelIntercept', kjit, delintercept, kjpindex, index)
       !
       CALL histwrite(hist_id, 'SoilMoist', kjit, tot_watsoil_end, kjpindex, index)
       CALL histwrite(hist_id, 'SoilWet', kjit, soilwet, kjpindex, index)
       !
       CALL histwrite(hist_id, 'RootMoist', kjit, tot_watsoil_end, kjpindex, index)
       CALL histwrite(hist_id, 'SubSnow', kjit, vevapsno, kjpindex, index)
       !
       CALL histwrite(hist_id, 'SnowDepth', kjit, snowdepth, kjpindex, index)
       !
    ENDIF
    !
    IF (long_print) WRITE (numout,*) ' hydrol_main Done '

  END SUBROUTINE hydrol_main

  !! Algorithm:
  !! - dynamic allocation for local array 
  !! - _restart_ file reading for HYDROLOGIC variables
  !!
  SUBROUTINE hydrol_init(kjit, ldrestart_read, kjpindex, index, rest_id, veget, humrel, vegstress, hdry,&
       & snow, snow_age, snow_nobio, snow_nobio_age, qsintveg)

    ! interface description
    ! input scalar 
    INTEGER(long), INTENT (in)                         :: kjit               !! Time step number 
    LOGICAL,INTENT (in)                                :: ldrestart_read     !! Logical for _restart_ file to read
    INTEGER(long), INTENT (in)                         :: kjpindex           !! Domain size
    INTEGER(long),DIMENSION (kjpindex), INTENT (in)    :: index              !! Indeces of the points on the map
    INTEGER(long), INTENT (in)                         :: rest_id            !! _Restart_ file identifier 
    ! input fields
    REAL(stnd),DIMENSION (kjpindex,nvm), INTENT (in)   :: veget              !! Carte de vegetation
    ! output fields
    REAL(stnd),DIMENSION (kjpindex,nvm), INTENT (out)  :: humrel             !! Stress hydrique, relative humidity
    REAL(stnd),DIMENSION (kjpindex,nvm), INTENT (out) :: vegstress     !! Veg. moisture stress (only for vegetation growth)
    REAL(stnd),DIMENSION (kjpindex), INTENT (out)      :: hdry               !! Dry soil heigth in meters
    REAL(stnd),DIMENSION (kjpindex), INTENT (out)      :: snow               !! Snow mass [Kg/m^2]
    REAL(stnd),DIMENSION (kjpindex), INTENT (out)      :: snow_age           !! Snow age
    REAL(stnd),DIMENSION (kjpindex,nnobio), INTENT (out) :: snow_nobio       !! Snow on ice, lakes, ...
    REAL(stnd),DIMENSION (kjpindex,nnobio), INTENT (out) :: snow_nobio_age   !! Snow age on ice, lakes, ...
    REAL(stnd),DIMENSION (kjpindex,nvm), INTENT (out)  :: qsintveg           !! Water on vegetation due to interception

    ! local declaration
    INTEGER(short)                                     :: ier,ipdt, jk
    INTEGER(short)                                     :: ji,jv
    REAL(stnd), DIMENSION (kjpindex,nvm)               :: zdsp

    ! initialisation
    IF (l_first_hydrol) THEN 
        l_first_hydrol=.FALSE.
    ELSE 
        WRITE (numout,*) ' l_first_hydrol false . we stop '
        STOP 'hydrol_init'
    ENDIF

    !Config  Key  = HYDROL_OK_HDIFF
    !Config  Desc = do horizontal diffusion?
    !Config  Def  = n
    !Config  Help = If TRUE, then water can diffuse horizontally between
    !Config         the PFTs' water reservoirs.
 
    ok_hdiff = .FALSE.
    CALL getin('HYDROL_OK_HDIFF',ok_hdiff) 

    ! make dynamic allocation with good dimension

    ! one dimension array allocation with possible restart value

    ALLOCATE (bqsb(kjpindex,nvm),stat=ier)
    IF (ier.NE.0) THEN
        WRITE (numout,*) ' error in bqsb allocation. We stop. We need kjpindex words = ',kjpindex
        STOP 'hydrol_init'
    END IF

    ALLOCATE (gqsb(kjpindex,nvm),stat=ier)
    IF (ier.NE.0) THEN
        WRITE (numout,*) ' error in gqsb allocation. We stop. We need kjpindex words = ',kjpindex
        STOP 'hydrol_init'
    END IF

    ALLOCATE (dsg(kjpindex,nvm),stat=ier)
    IF (ier.NE.0) THEN
        WRITE (numout,*) ' error in dsg allocation. We stop. We need kjpindex words = ',kjpindex
        STOP 'hydrol_init'
    END IF

    ALLOCATE (dsp(kjpindex,nvm),stat=ier)
    IF (ier.NE.0) THEN
        WRITE (numout,*) ' error in dsp allocation. We stop. We need kjpindex words = ',kjpindex
        STOP 'hydrol_init'
    END IF

    ! one dimension array allocation 

    ALLOCATE (mean_bqsb(kjpindex),stat=ier)
    IF (ier.NE.0) THEN
        WRITE (numout,*) ' error in mean_bqsb allocation. We stop. We need kjpindex words = ',kjpindex
        STOP 'hydrol_init'
    END IF

    ALLOCATE (mean_gqsb(kjpindex),stat=ier)
    IF (ier.NE.0) THEN
        WRITE (numout,*) ' error in mean_gqsb allocation. We stop. We need kjpindex words = ',kjpindex
        STOP 'hydrol_init'
    END IF

    ALLOCATE (dss(kjpindex,nvm),stat=ier)
    IF (ier.NE.0) THEN
        WRITE (numout,*) ' error in dss allocation. We stop. We need kjpindex words = ',kjpindex
        STOP 'hydrol_init'
    END IF

    ALLOCATE (precisol(kjpindex,nvm),stat=ier)
    IF (ier.NE.0) THEN
        WRITE (numout,*) ' error in precisol allocation. We stop. We need kjpindex words = ',kjpindex
        STOP 'hydrol_init'
    END IF

    ALLOCATE (gdrainage(kjpindex,nvm),stat=ier)
    IF (ier.NE.0) THEN
        WRITE (numout,*) ' error in precisol allocation. We stop. We need kjpindex words = ',kjpindex
        STOP 'hydrol_init'
    END IF

    ALLOCATE (subsnowveg(kjpindex),stat=ier)
    IF (ier.NE.0) THEN
        WRITE (numout,*) ' error in subsnowveg allocation. We stop. We need kjpindex words = ',kjpindex
        STOP 'hydrol_init'
    END IF

    ALLOCATE (subsnownobio(kjpindex,nnobio),stat=ier)
    IF (ier.NE.0) THEN
        WRITE (numout,*) ' error in subsnownobio allocation. We stop. We need kjpindex*nnobio words = ', &
          kjpindex*nnobio
        STOP 'hydrol_init'
    END IF

    ALLOCATE (snowmelt(kjpindex),stat=ier)
    IF (ier.NE.0) THEN
        WRITE (numout,*) ' error in snowmelt allocation. We stop. We need kjpindex words = ',kjpindex
        STOP 'hydrol_init'
    END IF

    ALLOCATE (icemelt(kjpindex),stat=ier)
    IF (ier.NE.0) THEN
        WRITE (numout,*) ' error in icemelt allocation. We stop. We need kjpindex words = ',kjpindex
        STOP 'hydrol_init'
    END IF

    ALLOCATE (mx_eau_var(kjpindex),stat=ier)
    IF (ier.NE.0) THEN
        WRITE (numout,*) ' error in mx_eau_var allocation. We stop. We need kjpindex words = ',kjpindex
        STOP 'hydrol_init'
    END IF

    ALLOCATE (ruu_ch(kjpindex),stat=ier)
    IF (ier.NE.0) THEN
        WRITE (numout,*) ' error in ruu_ch allocation. We stop. We need kjpindex words = ',kjpindex
        STOP 'hydrol_init'
    END IF

    ALLOCATE (vegtot(kjpindex),stat=ier)
    IF (ier.NE.0) THEN
        WRITE (numout,*) ' error in vegtot allocation. We stop. We need kjpindex words = ',kjpindex*nvm
        STOP 'hydrol_init'
    END IF

    ALLOCATE (resdist(kjpindex,nvm),stat=ier)
    IF (ier.NE.0) THEN
        WRITE (numout,*) ' error in resdist allocation. We stop. We need kjpindex words = ',kjpindex*nvm
        STOP 'hydrol_init'
    END IF

    ALLOCATE (runoff(kjpindex,nvm),stat=ier)
    IF (ier.NE.0) THEN
        WRITE (numout,*) ' error in runoff allocation. We stop. We need kjpindex words = ',kjpindex*nvm
        STOP 'hydrol_init'
    END IF
    !
    !  If we check the water balance we need two more variables
    !
    IF ( check_waterbal ) THEN

       ALLOCATE (tot_water_beg(kjpindex),stat=ier)
       IF (ier.NE.0) THEN
          WRITE (numout,*) ' error in tot_water_beg allocation. We stop. We need kjpindex words = ',kjpindex
          STOP 'hydrol_init'
       END IF

       ALLOCATE (tot_water_end(kjpindex),stat=ier)
       IF (ier.NE.0) THEN
          WRITE (numout,*) ' error in tot_water_end allocation. We stop. We need kjpindex words = ',kjpindex
          STOP 'hydrol_init'
       END IF

    ENDIF
    !
    !  If we use the almaoutputs we need four more variables
    !
    IF ( almaoutput ) THEN

       ALLOCATE (tot_watveg_beg(kjpindex),stat=ier)
       IF (ier.NE.0) THEN
          WRITE (numout,*) ' error in tot_watveg_beg allocation. We stop. We need kjpindex words = ',kjpindex
          STOP 'hydrol_init'
       END IF

       ALLOCATE (tot_watveg_end(kjpindex),stat=ier)
       IF (ier.NE.0) THEN
          WRITE (numout,*) ' error in tot_watveg_end allocation. We stop. We need kjpindex words = ',kjpindex
          STOP 'hydrol_init'
       END IF

       ALLOCATE (tot_watsoil_beg(kjpindex),stat=ier)
       IF (ier.NE.0) THEN
          WRITE (numout,*) ' error in tot_watsoil_beg allocation. We stop. We need kjpindex words = ',kjpindex
          STOP 'hydrol_init'
       END IF

       ALLOCATE (tot_watsoil_end(kjpindex),stat=ier)
       IF (ier.NE.0) THEN
          WRITE (numout,*) ' error in tot_watsoil_end allocation. We stop. We need kjpindex words = ',kjpindex
          STOP 'hydrol_init'
       END IF

       ALLOCATE (delsoilmoist(kjpindex),stat=ier)
       IF (ier.NE.0) THEN
          WRITE (numout,*) ' error in delsoilmoist allocation. We stop. We need kjpindex words = ',kjpindex
          STOP 'hydrol_init'
       END IF

       ALLOCATE (delintercept(kjpindex),stat=ier)
       IF (ier.NE.0) THEN
          WRITE (numout,*) ' error in delintercept. We stop. We need kjpindex words = ',kjpindex
          STOP 'hydrol_init'
       END IF

       ALLOCATE (delswe(kjpindex),stat=ier)
       IF (ier.NE.0) THEN
          WRITE (numout,*) ' error in delswe. We stop. We need kjpindex words = ',kjpindex
          STOP 'hydrol_init'
       ENDIF

       ALLOCATE (snow_beg(kjpindex),stat=ier)
       IF (ier.NE.0) THEN
          WRITE (numout,*) ' error in snow_beg allocation. We stop. We need kjpindex words =',kjpindex
          STOP 'hydrol_init'
       END IF

       ALLOCATE (snow_end(kjpindex),stat=ier)
       IF (ier.NE.0) THEN
          WRITE (numout,*) ' error in snow_end allocation. We stop. We need kjpindex words =',kjpindex
          STOP 'hydrol_init'
       END IF

    ENDIF

    ! open restart input file done by sechiba_init
    ! and read data from restart input file for HYDROLOGIC process

    IF (ldrestart_read) THEN

        IF (long_print) WRITE (numout,*) ' we have to read a restart file for HYDROLOGIC variables'

        var_name= 'snow'        
        CALL ioconf_setatt('UNITS', 'kg/m^2')
        CALL ioconf_setatt('LONG_NAME','Snow mass')
        CALL restget (rest_id, var_name, kjpindex, 1  , 1, kjit, .TRUE., snow, "gather", kjpindex, index)
        !
        var_name= 'snow_age'
        CALL ioconf_setatt('UNITS', 'd')
        CALL ioconf_setatt('LONG_NAME','Snow age')
        CALL restget (rest_id, var_name, kjpindex, 1  , 1, kjit, .TRUE., snow_age, "gather", kjpindex, index)
        !
        var_name= 'snow_nobio'
        CALL ioconf_setatt('UNITS', 'kg/m^2')
        CALL ioconf_setatt('LONG_NAME','Snow on other surface types')
        CALL restget (rest_id, var_name, kjpindex, nnobio  , 1, kjit, .TRUE., snow_nobio, "gather", kjpindex, index)
        !
        var_name= 'snow_nobio_age'
        CALL ioconf_setatt('UNITS', 'd')
        CALL ioconf_setatt('LONG_NAME','Snow age on other surface types')
        CALL restget (rest_id, var_name, kjpindex, nnobio  , 1, kjit, .TRUE., snow_nobio_age, "gather", kjpindex, index)
        !
        var_name= 'hdry'
        CALL ioconf_setatt('UNITS', 'm')
        CALL ioconf_setatt('LONG_NAME','Dry soil height')
        CALL restget (rest_id, var_name, kjpindex, 1  , 1, kjit, .TRUE., hdry, "gather", kjpindex, index)
        !
        var_name= 'humrel'
        CALL ioconf_setatt('UNITS', '-')
        CALL ioconf_setatt('LONG_NAME','Soil moisture stress')
        CALL restget (rest_id, var_name, kjpindex, nvm, 1, kjit, .TRUE., humrel, "gather", kjpindex, index)
        !
        var_name= 'vegstress'
        CALL ioconf_setatt('UNITS', '-')
        CALL ioconf_setatt('LONG_NAME','Vegetation growth moisture stress')
        CALL restget (rest_id, var_name, kjpindex, nvm, 1, kjit, .TRUE., vegstress, "gather", kjpindex, index)
        !
        var_name= 'bqsb'
        CALL ioconf_setatt('UNITS', 'kg/m^2')
        CALL ioconf_setatt('LONG_NAME','Deep soil moisture')
        CALL restget (rest_id, var_name, kjpindex, nvm , 1, kjit, .TRUE., bqsb, "gather", kjpindex, index)
        !
        var_name= 'gqsb'
        CALL ioconf_setatt('UNITS', 'kg/m^2')
        CALL ioconf_setatt('LONG_NAME','Surface soil moisture')
        CALL restget (rest_id, var_name, kjpindex, nvm , 1, kjit, .TRUE., gqsb, "gather", kjpindex, index)
        !
        var_name= 'dsg'
        CALL ioconf_setatt('UNITS', 'm')
        CALL ioconf_setatt('LONG_NAME','Depth of upper reservoir')
        CALL restget (rest_id, var_name, kjpindex, nvm  , 1, kjit, .TRUE., dsg, "gather", kjpindex, index)
        !
        var_name= 'dsp'
        CALL ioconf_setatt('UNITS', 'm')
        CALL ioconf_setatt('LONG_NAME','Depth to lower reservoir')
        CALL restget (rest_id, var_name, kjpindex, nvm  , 1, kjit, .TRUE., dsp, "gather", kjpindex, index)
        !
        var_name= 'qsintveg'
        CALL ioconf_setatt('UNITS', 'kg/m^2')
        CALL ioconf_setatt('LONG_NAME','Intercepted moisture')
        CALL restget (rest_id, var_name, kjpindex, nvm, 1, kjit, .TRUE., qsintveg, "gather", kjpindex, index)
        !
        var_name= 'resdist'
        CALL ioconf_setatt('UNITS', '-')
        CALL ioconf_setatt('LONG_NAME','Distribution of reservoirs')
        CALL restget (rest_id, var_name, kjpindex, nvm, 1, kjit, .TRUE., resdist, "gather", kjpindex, index)
        !
        ! get restart values if non were found in the restart file
        !
        !Config Key  = HYDROL_SNOW
        !Config Desc = Initial snow mass if not found in restart
        !Config Def  = 0.0
        !Config Help = The initial value of snow mass if its value is not found
        !Config        in the restart file. This should only be used if the model is 
        !Config        started without a restart file.
        !
        CALL setvar (snow, val_exp, 'HYDROL_SNOW', 0.0_stnd)
        !
        !Config Key  = HYDROL_SNOWAGE
        !Config Desc = Initial snow age if not found in restart
        !Config Def  = 0.0
        !Config Help = The initial value of snow age if its value is not found
        !Config        in the restart file. This should only be used if the model is 
        !Config        started without a restart file.
        !
        CALL setvar (snow_age, val_exp, 'HYDROL_SNOWAGE', 0.0_stnd)
        !
        !Config Key  = HYDROL_SNOW_NOBIO
        !Config Desc = Initial snow amount on ice, lakes, etc. if not found in restart
        !Config Def  = 0.0
        !Config Help = The initial value of snow if its value is not found
        !Config        in the restart file. This should only be used if the model is 
        !Config        started without a restart file.
        !
        CALL setvar (snow_nobio, val_exp, 'HYDROL_SNOW_NOBIO', 0.0_stnd)
        !
        !Config Key  = HYDROL_SNOW_NOBIO_AGE
        !Config Desc = Initial snow age on ice, lakes, etc. if not found in restart
        !Config Def  = 0.0
        !Config Help = The initial value of snow age if its value is not found
        !Config        in the restart file. This should only be used if the model is 
        !Config        started without a restart file.
        !
        CALL setvar (snow_nobio_age, val_exp, 'HYDROL_SNOW_NOBIO_AGE', 0.0_stnd)
        !
        !Config Key  = HYDROL_HDRY
        !Config Desc = Initial dry soil height if not found in restart
        !Config Def  = 0.0
        !Config Help = The initial value of dry soil height if its value is not found
        !Config        in the restart file. This should only be used if the model is 
        !Config        started without a restart file.
        !
        CALL setvar (hdry, val_exp, 'HYDROL_HDRY', 0.0_stnd)
        !
        !Config Key  = HYDROL_HUMR
        !Config Desc = Initial soil moisture stress if not found in restart
        !Config Def  = 1.0
        !Config Help = The initial value of soil moisture stress if its value is not found
        !Config        in the restart file. This should only be used if the model is 
        !Config        started without a restart file.
        !
        CALL setvar (humrel, val_exp,'HYDROL_HUMR', 1.0_stnd)
        CALL setvar (vegstress, val_exp,'HYDROL_HUMR', 1.0_stnd)
        !
        !Config Key  = HYDROL_BQSB
        !Config Desc = Initial restart deep soil moisture if not found in restart
        !Config Def  = DEF
        !Config Help = The initial value of deep soil moisture if its value is not found
        !Config        in the restart file. This should only be used if the model is 
        !Config        started without a restart file. Default behaviour is a saturated soil.
        !
        CALL setvar (bqsb, val_exp, 'HYDROL_BQSB', mx_eau_eau*dpu)
        !
        !Config Key  = HYDROL_GQSB
        !Config Desc = Initial upper soil moisture if not found in restart
        !Config Def  = 0.0
        !Config Help = The initial value of upper soil moisture if its value is not found
        !Config        in the restart file. This should only be used if the model is 
        !Config        started without a restart file.
        !
        CALL setvar (gqsb, val_exp, 'HYDROL_GQSB', 0.0_stnd)
        !
        !Config Key  = HYDROL_DSG
        !Config Desc = Initial upper reservoir depth if not found in restart
        !Config Def  = 0.0
        !Config Help = The initial value of upper reservoir depth if its value is not found
        !Config        in the restart file. This should only be used if the model is 
        !Config        started without a restart file.
        !
        CALL setvar (dsg, val_exp, 'HYDROL_DSG', 0.0_stnd)

        ! set inital value for dsp if needed
        !
        !Config Key  = HYDROL_DSP
        !Config Desc = Initial dry soil above upper reservoir if not found in restart
        !Config Def  = DEF
        !Config Help = The initial value of dry soil above upper reservoir if its value 
        !Config        is not found in the restart file. This should only be used if
        !Config        the model is started without a restart file. The default behaviour
        !Config        is to compute it from the variables above. Should be OK most of 
        !Config        the time.
        !
        zdsp(:,:) = dpu - bqsb(:,:) / mx_eau_eau
        CALL setvar (dsp, val_exp, 'HYDROL_DSP', zdsp)
        !
        !Config Key  = HYDROL_QSV
        !Config Desc = Initial water on canopy if not found in restart
        !Config Def  = 0.0
        !Config Help = The initial value of moisture on canopy if its value 
        !Config        is not found in the restart file. This should only be used if
        !Config        the model is started without a restart file. 
        !
        CALL setvar (qsintveg, val_exp, 'HYDROL_QSV', 0.0_stnd)
        dss = dsg - gqsb / mx_eau_eau
        !
        ! There is no need to configure the initialisation of resdist. If not available it is the vegetation map
        !
        IF ( MINVAL(resdist) .EQ.  MAXVAL(resdist) .AND. MINVAL(resdist) .EQ. val_exp) THEN
            resdist = veget
        ENDIF
        !
        !  Remember that it is only frac_nobio + SUM(veget(,:)) that is equal to 1. Thus we need vegtot
        !
        DO jk = 1, kjpindex
           vegtot(jk) = SUM(veget(jk,:))
        ENDDO
        !
    ENDIF

    !
    ! Where vegetation fraction is zero, set water to that of bare soil.
    ! This does not create any additional water.
    !
    DO jv = 2, nvm
      DO ji = 1, kjpindex
        IF ( veget(ji,jv) .LT. EPSILON(un) ) THEN
          gqsb(ji,jv) = gqsb(ji,1)
          bqsb(ji,jv) = bqsb(ji,1)
          dsg(ji,jv) = dsg(ji,1)
          dss(ji,jv) = dss(ji,1)
          dsp(ji,jv) = dsp(ji,1)
        ENDIF
      ENDDO
    ENDDO
    !
    !
    CALL getin('SECHIBA_QSINT', qsintcst)
    print*,'DANS HYDROL_INIT, SECHIBA_QSINT=',qsintcst


    IF (long_print) WRITE (numout,*) ' hydrol_init done '
    !
  END SUBROUTINE hydrol_init
  !
  !-------------------------------------
  !
  SUBROUTINE hydrol_clear()
 
    l_first_hydrol=.TRUE.

    IF (ALLOCATED (bqsb)) DEALLOCATE (bqsb)
    IF (ALLOCATED  (gqsb)) DEALLOCATE (gqsb)
    IF (ALLOCATED  (dsg))  DEALLOCATE (dsg)
    IF (ALLOCATED  (dsp))  DEALLOCATE (dsp)
    IF (ALLOCATED  (mean_bqsb))  DEALLOCATE (mean_bqsb)
    IF (ALLOCATED  (mean_gqsb)) DEALLOCATE (mean_gqsb)
    IF (ALLOCATED  (dss)) DEALLOCATE (dss)
    IF (ALLOCATED  (precisol)) DEALLOCATE (precisol)
    IF (ALLOCATED  (gdrainage)) DEALLOCATE (gdrainage)
    IF (ALLOCATED  (subsnowveg)) DEALLOCATE (subsnowveg)
    IF (ALLOCATED  (subsnownobio)) DEALLOCATE (subsnownobio)
    IF (ALLOCATED  (snowmelt)) DEALLOCATE (snowmelt)
    IF (ALLOCATED  (icemelt)) DEALLOCATE (icemelt)
    IF (ALLOCATED  (mx_eau_var)) DEALLOCATE (mx_eau_var)
    IF (ALLOCATED  (ruu_ch)) DEALLOCATE (ruu_ch)
    IF (ALLOCATED  (vegtot)) DEALLOCATE (vegtot)
    IF (ALLOCATED  (resdist)) DEALLOCATE (resdist)
    IF (ALLOCATED  (runoff)) DEALLOCATE (runoff)
    IF (ALLOCATED  (tot_water_beg)) DEALLOCATE (tot_water_beg)
    IF (ALLOCATED  (tot_water_end)) DEALLOCATE (tot_water_end)
    IF (ALLOCATED  (tot_watveg_beg)) DEALLOCATE (tot_watveg_beg)
    IF (ALLOCATED  (tot_watveg_end)) DEALLOCATE (tot_watveg_end)
    IF (ALLOCATED  (tot_watsoil_beg)) DEALLOCATE (tot_watsoil_beg)
    IF (ALLOCATED  (tot_watsoil_end)) DEALLOCATE (tot_watsoil_end)
    IF (ALLOCATED  (delsoilmoist)) DEALLOCATE (delsoilmoist)
    IF (ALLOCATED  (delintercept)) DEALLOCATE (delintercept)
    IF (ALLOCATED  (snow_beg)) DEALLOCATE (snow_beg)
    IF (ALLOCATED  (snow_end)) DEALLOCATE (snow_end)
    IF (ALLOCATED  (delswe)) DEALLOCATE (delswe)
    !
 END SUBROUTINE hydrol_clear

  !! This routine initializes HYDROLOGIC variables 
  !! - mx_eau_var
  !! - ruu_ch
  !!
  SUBROUTINE hydrol_var_init (kjpindex, veget, veget_max, hdry, mx_eau_var, ruu_ch, shumdiag, litterhumdiag)

    ! interface description
    ! input scalar 
    INTEGER(long), INTENT(in)                         :: kjpindex      !! Domain size
    ! input fields
    REAL(stnd),DIMENSION (kjpindex,nvm), INTENT (in)  :: veget         !! Fraction of vegetation type 
    REAL(stnd),DIMENSION (kjpindex,nvm), INTENT (in)  :: veget_max     !! Max. fraction of vegetation type 
    REAL(stnd),DIMENSION (kjpindex), INTENT (in)      :: hdry          !! Dry soil heigth in meters
    ! output fields
    !! Profondeur du reservoir contenant le maximum d'eau  
    REAL(stnd),DIMENSION (kjpindex), INTENT (out)     :: mx_eau_var    !!
    REAL(stnd),DIMENSION (kjpindex), INTENT (out)     :: ruu_ch        !! Quantite d'eau maximum
    REAL(stnd),DIMENSION (kjpindex,nbdl), INTENT (out):: shumdiag      !! relative soil moisture
    REAL(stnd),DIMENSION (kjpindex), INTENT (out)     :: litterhumdiag !! litter humidity
    ! local declaration
    INTEGER(short)                                    :: ji,jv, jd
    REAL(stnd), DIMENSION(kjpindex)                   :: mean_dsg
    REAL(stnd)                                        :: gtr, btr
    REAL(stnd), DIMENSION(nbdl+1)                     :: tmp_dl
    !
    ! initialisation
    tmp_dl(1) = 0
    tmp_dl(2:nbdl+1) = diaglev(1:nbdl)
    !
    mx_eau_var(:) = 0.0
    !
    DO ji = 1,kjpindex
      DO jv = 1,nvm
         mx_eau_var(ji) = mx_eau_var(ji) + veget(ji,jv)*wmax_veg(jv)*dpu
      END DO
      IF (vegtot(ji) .GT. zero) THEN
         mx_eau_var(ji) = mx_eau_var(ji)/vegtot(ji)
      ELSE
         mx_eau_var(ji) = mx_eau_eau*dpu
      ENDIF
      ruu_ch(ji) = mx_eau_var(ji) / dpu
    END DO
    !
    !
    ! could be done with SUM instruction but this kills vectorization
    mean_bqsb(:) = 0.0
    mean_gqsb(:) = 0.0
    mean_dsg(:) = 0.0
    DO jv = 1, nvm
      DO ji = 1, kjpindex
        mean_bqsb(ji) = mean_bqsb(ji) + resdist(ji,jv)*bqsb(ji,jv)
        mean_gqsb(ji) = mean_gqsb(ji) + resdist(ji,jv)*gqsb(ji,jv)
        mean_dsg(ji) = mean_dsg(ji) + resdist(ji,jv)*dsg(ji,jv)
      ENDDO
    ENDDO
    mean_dsg(:) = MAX( mean_dsg(:), mean_gqsb(:)/ruu_ch(:) )

    DO ji = 1, kjpindex
      IF (vegtot(ji) .GT. zero) THEN
        mean_bqsb(ji) = mean_bqsb(ji)/vegtot(ji)
        mean_gqsb(ji) = mean_gqsb(ji)/vegtot(ji)
        mean_dsg(ji) = mean_dsg(ji)/vegtot(ji)
      ENDIF
    ENDDO

    DO ji = 1,kjpindex
       !
       DO jd = 1,nbdl
          IF ( tmp_dl(jd+1) .LT. mean_dsg(ji)) THEN
             shumdiag(ji,jd) = mean_gqsb(ji)/mx_eau_var(ji)
          ELSE
             IF ( tmp_dl(jd) .LT. mean_dsg(ji)) THEN
                gtr = (mean_dsg(ji)-tmp_dl(jd))/(tmp_dl(jd+1)-tmp_dl(jd))
                btr = 1 - gtr
                shumdiag(ji,jd) = gtr*mean_gqsb(ji)/mx_eau_var(ji) + &
                     & btr*mean_bqsb(ji)/mx_eau_var(ji)
             ELSE
                shumdiag(ji,jd) = mean_bqsb(ji)/mx_eau_var(ji)
             ENDIF
          ENDIF
          shumdiag(ji,jd) = MAX(MIN(shumdiag(ji,jd), un), zero)
       ENDDO
    ENDDO

    !
    ! litter humidity.
    !

    DO ji = 1, kjpindex
      litterhumdiag(ji) = EXP( - hdry(ji) / hcrit_litter )
    ENDDO

    ! special case: it has just been raining a few drops. The upper soil
    ! reservoir is ridiculously small, but the dry soil height is zero.
    ! Don't take it into account.

    DO ji = 1, kjpindex
      IF ( ( hdry(ji) .LT. min_sechiba ) .AND. &
            ( mean_dsg(ji) .GT. min_sechiba ) .AND. &
            ( mean_dsg(ji) .LT. 5.E-4 ) ) THEN
        litterhumdiag(ji) = 0.0
      ENDIF
    ENDDO
    
    IF (long_print) WRITE (numout,*) ' hydrol_var_init done '

  END SUBROUTINE hydrol_var_init

  !! This routine computes snow processes
  !!
  SUBROUTINE hydrol_snow (kjpindex, dtradia, precip_rain, precip_snow , temp_sol_new, soilcap,&
       & frac_nobio, totfrac_nobio, vevapnu, vevapsno, snow, snow_age, snow_nobio, snow_nobio_age, &
       & tot_melt, snowdepth)

    ! 
    ! interface description
    ! input scalar 
    INTEGER(long), INTENT(in)                               :: kjpindex      !! Domain size
    REAL(stnd), INTENT (in)                                 :: dtradia       !! Time step in seconds
    ! input fields
    REAL(stnd), DIMENSION (kjpindex), INTENT(in)            :: precip_rain   !! Rainfall
    REAL(stnd), DIMENSION (kjpindex), INTENT(in)            :: precip_snow   !! Snow precipitation
    REAL(stnd), DIMENSION (kjpindex), INTENT(in)            :: temp_sol_new  !! New soil temperature
    REAL(stnd), DIMENSION (kjpindex), INTENT(in)            :: soilcap       !! Soil capacity
    REAL(stnd), DIMENSION (kjpindex,nnobio), INTENT(in)     :: frac_nobio    !! Fraction of continental ice, lakes, ...
    REAL(stnd), DIMENSION (kjpindex), INTENT(in)            :: totfrac_nobio !! Total fraction of continental ice+lakes+ ...
    ! modified fields
    REAL(stnd), DIMENSION (kjpindex), INTENT(inout)         :: vevapnu       !! Bare soil evaporation
    REAL(stnd), DIMENSION (kjpindex), INTENT(inout)         :: vevapsno      !! Snow evaporation
    REAL(stnd), DIMENSION (kjpindex), INTENT(inout)         :: snow          !! Snow mass [Kg/m^2]
    REAL(stnd), DIMENSION (kjpindex), INTENT(inout)         :: snow_age      !! Snow age
    REAL(stnd), DIMENSION (kjpindex,nnobio), INTENT(inout)  :: snow_nobio    !! Ice water balance
    REAL(stnd), DIMENSION (kjpindex,nnobio), INTENT(inout)  :: snow_nobio_age!! Snow age on ice, lakes, ...
    ! output fields
    REAL(stnd), DIMENSION (kjpindex), INTENT(out)           :: tot_melt      !! Total melt  
    REAL(stnd), DIMENSION (kjpindex), INTENT(out)           :: snowdepth     !! Snow depth 
    !
    ! local declaration
    !
    INTEGER(short)                               :: ji, jv
    REAL(stnd), DIMENSION (kjpindex)             :: d_age  !! Snow age change
    REAL(stnd), DIMENSION (kjpindex)             :: xx     !! temporary
    REAL(stnd)                                   :: snowmelt_tmp !! The name says it all !

    !
    ! for continental points
    !

    ! 
    ! 0. initialisation
    !
    DO jv = 1, nnobio
      DO ji=1,kjpindex
        subsnownobio(ji,jv) = zero
      ENDDO
    ENDDO
    DO ji=1,kjpindex
      subsnowveg(ji) = zero
      snowmelt(ji) = zero
      icemelt(ji) = zero
      tot_melt(ji) = zero
    ENDDO
    !
    ! 1. On vegetation
    !
    DO ji=1,kjpindex
      !
      ! 1.1. It is snowing
      !
      snow(ji) = snow(ji) + (un - totfrac_nobio(ji))*precip_snow(ji)
      !
      ! 1.2. Sublimation - separate between vegetated and no-veget fractions 
      !      Care has to be taken as we might have sublimation from the
      !      the frac_nobio while there is no snow on the rest of the grid.
      !
      IF ( snow(ji) > snowcri ) THEN
        subsnownobio(ji,iice) = frac_nobio(ji,iice)*vevapsno(ji)
        subsnowveg(ji) = vevapsno(ji) - subsnownobio(ji,iice)
      ELSE
        subsnownobio(ji,iice) = vevapsno(ji)
        subsnowveg(ji) = zero
      ENDIF
      !
      ! 1.2.1 Check that sublimation on the vegetated fraction is possible.
      !
      IF (subsnowveg(ji) .GT. snow(ji)) THEN 
         ! What could not be sublimated goes into bare soil evaporation
         vevapnu(ji) = vevapnu(ji) + (subsnowveg(ji) - snow(ji))
         ! Sublimation is thus limited to what is available
         subsnowveg(ji) = snow(ji)
         snow(ji) = zero
         vevapsno(ji) = subsnowveg(ji) + subsnownobio(ji,iice)
      ELSE 
         snow(ji) = snow(ji) - subsnowveg(ji)
      ENDIF
      !
      ! 1.3. snow melt only if temperature positive
      !
      IF (temp_sol_new(ji).GT.tp_00) THEN 
         !
         IF (snow(ji).GT.sneige) THEN 
            !
            snowmelt(ji) = (1. - frac_nobio(ji,iice))*(temp_sol_new(ji) - tp_00) * soilcap(ji) / chalfu0 
            !
            ! 1.3.1.1 enough snow for melting or not
            !
            IF (snowmelt(ji).LT.snow(ji)) THEN 
               snow(ji) = snow(ji) - snowmelt(ji)
            ELSE 
               snowmelt(ji) = snow(ji)
               snow(ji) = zero
            END IF
            !
         ELSEIF (snow(ji).GE.zero) THEN 
            !
            ! 1.3.2 not enough snow
            !
            snowmelt(ji) = snow(ji)
            snow(ji) = zero
         ELSE 
            !
            ! 1.3.3 negative snow - now snow melt
            !
            snow(ji) = zero
            snowmelt(ji) = zero
            WRITE(numout,*) 'hydrol_snow: WARNING! snow was negative and was reset to zero. '
            !
         END IF

      ENDIF

    END DO
    !
    ! 2. On Land ice
    !
    DO ji=1,kjpindex
      !
      ! 2.1. It is snowing
      !
      snow_nobio(ji,iice) = snow_nobio(ji,iice) + frac_nobio(ji,iice)*precip_snow(ji) + &
           & frac_nobio(ji,iice)*precip_rain(ji)
      !
      ! 2.2. Sublimation - was calculated before it can give us negative snow_nobio but that is OK
      !      Once it goes below a certain values (-maxmass_glacier for instance) we should kill
      !      the frac_nobio(ji,iice) !
      !
      snow_nobio(ji,iice) = snow_nobio(ji,iice) - subsnownobio(ji,iice)
      !
      ! 2.3. ice melt only for continental ice fraction
      !
      snowmelt_tmp = zero
      IF (temp_sol_new(ji) .GT. tp_00) THEN 
         !
         ! 2.3.1 If there is snow on the ice-fraction it can melt 
         !
         snowmelt_tmp = frac_nobio(ji,iice)*(temp_sol_new(ji) - tp_00) * soilcap(ji) / chalfu0
         !
         IF ( snowmelt_tmp .GT. snow_nobio(ji,iice) ) THEN
            snowmelt_tmp = MAX( 0., snow_nobio(ji,iice))
         ENDIF
         snowmelt(ji) = snowmelt(ji) + snowmelt_tmp
         snow_nobio(ji,iice) = snow_nobio(ji,iice) - snowmelt_tmp
         !
      ENDIF
      !
      ! Ice melt only if there is more than a given mass : maxmass_glacier, i.e. only weight melts glaciers !
      !
      IF ( snow_nobio(ji,iice) .GE. maxmass_glacier ) THEN
         icemelt(ji) = snow_nobio(ji,iice) - maxmass_glacier
         snow_nobio(ji,iice) = maxmass_glacier
      ENDIF
      !
    END DO

    !
    ! 3. On other surface types - not done yet
    !
    IF ( nnobio .GT. 1 ) THEN
      WRITE(*,*) 'WE HAVE',nnobio-1,' SURFACE TYPES I DO NOT KNOW'
      WRITE(*,*) 'CANNOT TREAT SNOW ON THESE SURFACE TYPES'
      STOP
    ENDIF

    !
    ! 4. computes total melt (snow and ice)
    !

    DO ji = 1, kjpindex
      tot_melt(ji) = icemelt(ji) + snowmelt(ji)
    ENDDO

    ! 
    ! 5. computes snow age on veg and ice (for albedo)
    !
    DO ji = 1, kjpindex
      !
      ! 5.1 Snow age on vegetation
      !
      IF (snow(ji) .LE. zero) THEN
        snow_age(ji) = zero
      ELSE
        snow_age(ji) =(snow_age(ji) + (un - snow_age(ji)/max_snow_age) * dtradia/un_jour) &
          & * EXP(-precip_snow(ji) / snow_trans)
      ENDIF
      !
      ! 5.2 Snow age on ice
      !
      ! age of snow on ice: a little bit different because in cold regions, we really
      ! cannot negect the effect of cold temperatures on snow metamorphism any more.
      !
      IF (snow_nobio(ji,iice) .LE. zero) THEN 
        snow_nobio_age(ji,iice) = zero
      ELSE
      !
        d_age(ji) = ( snow_nobio_age(ji,iice) + &
                    &  (un - snow_nobio_age(ji,iice)/max_snow_age) * dtradia/un_jour ) * &
                    &  EXP(-precip_snow(ji) / snow_trans) - snow_nobio_age(ji,iice)
        IF (d_age(ji) .GT. 0. ) THEN
          xx(ji) = MAX( tp_00 - temp_sol_new(ji), zero )
          xx(ji) = ( xx(ji) / 7._stnd ) ** 4._stnd
          d_age(ji) = d_age(ji) / (un+xx(ji))
        ENDIF
        snow_nobio_age(ji,iice) = MAX( snow_nobio_age(ji,iice) + d_age(ji), zero )
      !
      ENDIF

    ENDDO

    !
    ! 6.0 Diagnose the depth of the snow layer
    !

    DO ji = 1, kjpindex
      snowdepth(ji) = snow(ji) /sn_dens
    ENDDO

    IF (long_print) WRITE (numout,*) ' hydrol_snow done '

  END SUBROUTINE hydrol_snow

  !! This routine computes canopy processes
  !!
  SUBROUTINE hydrol_canop (kjpindex, precip_rain, vevapwet, veget, qsintmax, qsintveg, precisol)

    ! 
    ! interface description
    !
    INTEGER(long), INTENT(in)                               :: kjpindex    !! Domain size
    ! input fields
    REAL(stnd), DIMENSION (kjpindex), INTENT(in)            :: precip_rain !! Rain precipitation
    REAL(stnd), DIMENSION (kjpindex,nvm), INTENT(in)        :: vevapwet    !! Interception loss
    REAL(stnd), DIMENSION (kjpindex,nvm), INTENT(in)        :: veget       !! Fraction of vegetation type 
    REAL(stnd), DIMENSION (kjpindex,nvm), INTENT(in)        :: qsintmax    !! Maximum water on vegetation for interception
    ! modified fields
    REAL(stnd), DIMENSION (kjpindex,nvm), INTENT(inout)     :: qsintveg    !! Water on vegetation due to interception
    ! output fields
    REAL(stnd), DIMENSION (kjpindex,nvm), INTENT(out)           :: precisol    !! Eau tombee sur le sol

    !
    ! local declaration
    !
    INTEGER(short)                                :: ji, jv
    REAL(stnd), DIMENSION (kjpindex,nvm)          :: zqsintvegnew

    ! calcul de qsintmax a prevoir a chaque pas de temps
    ! dans ini_sechiba
    ! boucle sur les points continentaux
    ! calcul de qsintveg au pas de temps suivant
    ! par ajout du flux interception loss
    ! calcule par enerbil en fonction
    ! des calculs faits dans diffuco
    ! calcul de ce qui tombe sur le sol
    ! avec accumulation dans precisol
    ! essayer d'harmoniser le traitement du sol nu
    ! avec celui des differents types de vegetation
    ! fait si on impose qsintmax ( ,1) = 0.0
    !
    ! loop for continental subdomain
    !

    !
    ! 1. evaporation off the continents
    !
    ! 1.1 The interception loss is take off the canopy. 

    if (qsintcst.GT.min_sechiba) then

    DO jv=1,nvm
      qsintveg(:,jv) = qsintveg(:,jv) - vevapwet(:,jv)
    END DO

    ! 1.2 It is raining : precip_rain is shared for each vegetation
    ! type
    !     sum (veget (1,nvm)) must be egal to 1-totfrac_nobio.
    !     iniveget computes veget each day
    !
    DO jv=1,nvm
      qsintveg(:,jv) = qsintveg(:,jv) + veget(:,jv) * precip_rain(:)
    END DO

    !
    ! 1.3 Limits the effect and sum what receives soil
    !
    precisol(:,:) = zero
    DO jv=1,nvm
      DO ji = 1, kjpindex
        zqsintvegnew(ji,jv) = MIN (qsintveg(ji,jv),qsintmax(ji,jv)) 
        precisol(ji,jv) = qsintveg(ji,jv ) - zqsintvegnew (ji,jv)
      ENDDO
    ENDDO
    !
    ! 1.4 swap qsintveg to the new value
    !

    DO jv=1,nvm
      qsintveg(:,jv) = zqsintvegnew (:,jv)
    END DO

    else

! Modif FH
! Dans le cas ou le reservoir max sur les feuilles est nul, on ne
! traite pas l'interception

    DO jv=1,nvm
        zqsintvegnew(:,jv) = 0.
        qsintveg(:,jv) = 0.
        precisol(:,jv) =  veget(:,jv) * precip_rain(:)
    ENDDO

    endif

    IF (long_print) WRITE (numout,*) ' hydrol_canop done '

  END SUBROUTINE hydrol_canop
  !!
  !!
  !!
  SUBROUTINE hydrol_vegupd(kjpindex, veget, ruu_ch, qsintveg, gqsb, bqsb, dsg, dss, dsp, resdist)
    !
    !
    !   The vegetation cover has changed and we need to adapt the reservoir distribution.
    !   You may note that this occurs after evaporation and so on have been computed. It is 
    !   not a problem as a new vegetation fraction will start with humrel=0 and thus will have no
    !   evaporation. If this is not the case it should have been caught above.
    !
    ! input scalar 
    INTEGER(long), INTENT(in)                               :: kjpindex 
    ! input fields
    REAL(stnd), DIMENSION (kjpindex, nvm), INTENT(in)       :: veget            !! New vegetation map
    REAL(stnd), DIMENSION (kjpindex), INTENT(in)            :: ruu_ch           !! Quantite d'eau maximum
    ! modified fields
    REAL(stnd),DIMENSION (kjpindex,nvm), INTENT (inout)  :: qsintveg           !! Water on vegetation due to interception
    REAL(stnd), DIMENSION (kjpindex,nvm), INTENT(inout)     :: gqsb             !! Hauteur d'eau dans le reservoir de surface
    REAL(stnd), DIMENSION (kjpindex,nvm), INTENT(inout)     :: bqsb             !! Hauteur d'eau dans le reservoir profond
    REAL(stnd), DIMENSION (kjpindex,nvm), INTENT(inout)     :: dsg              !! Hauteur du reservoir de surface
    REAL(stnd), DIMENSION (kjpindex,nvm), INTENT(inout)     :: dss              !! Hauteur au dessus du reservoir de surface
    REAL(stnd), DIMENSION (kjpindex,nvm), INTENT(inout)     :: dsp              !! Hauteur au dessus du reservoir profond 
    REAL(stnd), DIMENSION (kjpindex, nvm), INTENT(inout)    :: resdist          !! Old vegetation map
    !
    !
    ! local declaration
    !
    INTEGER(short)                                :: ji,jv
    !
    REAL(stnd),DIMENSION (kjpindex,nvm)           :: qsintveg2
    REAL(stnd), DIMENSION (kjpindex,nvm)          :: bdq, gdq, qsdq
    REAL(stnd), DIMENSION (kjpindex,nvm)          :: vmr
    REAL(stnd), DIMENSION(kjpindex)               :: gtr, btr, vtr, qstr, fra
    REAL(stnd), DIMENSION(kjpindex) :: vegchtot
    !
    DO jv = 1, nvm
      DO ji = 1, kjpindex
        IF ( ABS(veget(ji,jv)-resdist(ji,jv)) .GT. EPSILON(un) ) THEN
          vmr(ji,jv) = veget(ji,jv)-resdist(ji,jv)
        ELSE
          vmr(ji,jv) = 0.0
        ENDIF
    !
        IF (resdist(ji,jv) .GT. 0.) THEN
         qsintveg2(ji,jv) = qsintveg(ji,jv)/resdist(ji,jv)
        ELSE
         qsintveg2(ji,jv) = zero
        ENDIF   
      ENDDO
    ENDDO
    !
    vegchtot(:) = 0.
    DO jv = 1, nvm
      DO ji = 1, kjpindex
        vegchtot(ji) = vegchtot(ji) + ABS( vmr(ji,jv) )
      ENDDO
    ENDDO
    !
    DO jv = 1, nvm
      DO ji = 1, kjpindex
        IF ( vegchtot(ji) .GT. 0. ) THEN
          gdq(ji,jv) = ABS(vmr(ji,jv)) * gqsb(ji,jv)
          bdq(ji,jv) = ABS(vmr(ji,jv)) * bqsb(ji,jv)
          qsdq(ji,jv) = ABS(vmr(ji,jv)) * qsintveg2(ji,jv)
        ENDIF
      ENDDO
    ENDDO
    !
    ! calculate water mass that we have to redistribute
    !
    gtr(:) = zero
    btr(:) = zero
    qstr(:) = zero
    vtr(:) = zero
    !
    !
    DO jv = 1, nvm
      DO ji = 1, kjpindex
        IF ( ( vegchtot(ji) .GT. 0. ) .AND. ( vmr(ji,jv) .LT. 0. ) ) THEN
          gtr(ji) = gtr(ji) + gdq(ji,jv)
          btr(ji) = btr(ji) + bdq(ji,jv)
          qstr(ji) = qstr(ji) + qsdq(ji,jv) 
          vtr(ji) = vtr(ji) - vmr(ji,jv)
        ENDIF
      ENDDO
    ENDDO
    !
    ! put it into reservoir of plant whose surface area has grown
    DO jv = 1, nvm
      DO ji = 1, kjpindex
        IF ( vegchtot(ji) .GT. 0. .AND. ABS(vtr(ji)) .GT. EPSILON(un)) THEN
            fra(ji) = vmr(ji,jv) / vtr(ji)
             IF ( vmr(ji,jv) .GT. 0.)  THEN
              IF (veget(ji,jv) .GT. 0.) THEN
               gqsb(ji,jv) = (resdist(ji,jv)*gqsb(ji,jv) + fra(ji)*gtr(ji))/veget(ji,jv)
               bqsb(ji,jv) = (resdist(ji,jv)*bqsb(ji,jv) + fra(ji)*btr(ji))/veget(ji,jv)
              ENDIF
              qsintveg(ji,jv) = qsintveg(ji,jv) + fra(ji)* qstr(ji)
             ELSE
              qsintveg(ji,jv) = qsintveg(ji,jv) - qsdq(ji,jv)
             ENDIF
             !
             ! dss is not altered so that this transfer of moisture does not directly 
             ! affect transpiration
             IF (gqsb(ji,jv) .LT. min_sechiba) THEN
                dsg(ji,jv) = 0.0
             ELSE
                dsg(ji,jv) = (dss(ji,jv) * ruu_ch(ji) + gqsb(ji,jv)) &
                             / ruu_ch(ji)
             ENDIF
             dsp(ji,jv) = dpu - bqsb(ji,jv) / ruu_ch(ji)
        ENDIF
      ENDDO
    ENDDO

    !   Now that the work is done resdist needs an update !
    DO jv = 1, nvm
      DO ji = 1, kjpindex
        resdist(ji,jv) = veget(ji,jv)
      ENDDO
    ENDDO

    !
    ! Where vegetation fraction is zero, set water to that of bare soil.
    ! This does not create any additional water.
    !
    DO jv = 2, nvm
      DO ji = 1, kjpindex
        IF ( veget(ji,jv) .LT. EPSILON(un) ) THEN
          gqsb(ji,jv) = gqsb(ji,1)
          bqsb(ji,jv) = bqsb(ji,1)
          dsg(ji,jv) = dsg(ji,1)
          dss(ji,jv) = dss(ji,1)
          dsp(ji,jv) = dsp(ji,1)
        ENDIF
      ENDDO
    ENDDO

    RETURN
    !
  END SUBROUTINE hydrol_vegupd
  !!
  !! This routines computes soil processes
  !!
  SUBROUTINE hydrol_soil(kjpindex, vevapnu, precisol, returnflow, irrigation, tot_melt, mx_eau_var, veget, ruu_ch, transpir,&
       & gqsb, bqsb, dsg, dss, hdry, dsp, runoff, run_off_tot, drainage, humrel, vegstress, shumdiag, litterhumdiag)
    ! 
    ! interface description
    ! input scalar 
    INTEGER(long), INTENT(in)                               :: kjpindex 
    ! input fields
    REAL(stnd), DIMENSION (kjpindex), INTENT(in)            :: vevapnu          !! Bare soil evaporation
    REAL(stnd), DIMENSION (kjpindex,nvm), INTENT(in)        :: precisol         !! Eau tombee sur le sol
    REAL(stnd), DIMENSION (kjpindex), INTENT(in)            :: returnflow       !! Water returning to the deep reservoir
    REAL(stnd), DIMENSION (kjpindex), INTENT(in)            :: irrigation       !! Irrigation
    REAL(stnd), DIMENSION (kjpindex), INTENT(in)            :: tot_melt         !! Total melt   
    REAL(stnd), DIMENSION (kjpindex), INTENT(in)            :: mx_eau_var       !! Profondeur du reservoir contenant le
                                                                                !! maximum d'eau  
    REAL(stnd), DIMENSION (kjpindex, nvm), INTENT(in)       :: veget            !! Vegetation map
    REAL(stnd), DIMENSION (kjpindex), INTENT(in)            :: ruu_ch           !! Quantite d'eau maximum
    REAL(stnd), DIMENSION (kjpindex,nvm), INTENT(in)        :: transpir         !! Transpiration
    ! modified fields
    REAL(stnd), DIMENSION (kjpindex,nvm), INTENT(inout)     :: gqsb             !! Hauteur d'eau dans le reservoir de surface
    REAL(stnd), DIMENSION (kjpindex,nvm), INTENT(inout)     :: bqsb             !! Hauteur d'eau dans le reservoir profond
    REAL(stnd), DIMENSION (kjpindex,nvm), INTENT(inout)     :: dsg              !! Hauteur du reservoir de surface
    REAL(stnd), DIMENSION (kjpindex,nvm), INTENT(inout)     :: dss              !! Hauteur au dessus du reservoir de surface
    REAL(stnd), DIMENSION (kjpindex,nvm), INTENT(inout)     :: dsp              !! Hauteur au dessus du reservoir profond 
    ! output fields
    REAL(stnd), DIMENSION (kjpindex,nvm), INTENT(out)       :: runoff           !! Ruissellement
    REAL(stnd), DIMENSION (kjpindex), INTENT(out)           :: run_off_tot      !! Complete runoff
    REAL(stnd), DIMENSION (kjpindex), INTENT(out)           :: drainage         !! Drainage
    REAL(stnd), DIMENSION (kjpindex, nvm), INTENT(out)      :: humrel           !! Relative humidity
    REAL(stnd), DIMENSION (kjpindex, nvm), INTENT(out)      :: vegstress   !! Veg. moisture stress (only for vegetation growth)
    REAL(stnd),DIMENSION (kjpindex,nbdl), INTENT (out)      :: shumdiag         !! relative soil moisture
    REAL(stnd),DIMENSION (kjpindex), INTENT (out)           :: litterhumdiag    !! litter humidity
    REAL(stnd), DIMENSION (kjpindex), INTENT(out)           :: hdry             !! Dry soil heigth in meters

! Modifs stabilite
    real dsseff,dsg_min
    REAL(stnd), DIMENSION(kjpindex)               :: a_subgrd

    !
    ! local declaration
    !
    INTEGER(short)                                :: ji,jv, jd
    REAL(stnd), DIMENSION(kjpindex)               :: zhumrel_lo, zhumrel_up
    REAL(stnd), DIMENSION(kjpindex,nvm)           :: zeflux, zpreci
    LOGICAL, DIMENSION(kjpindex,nvm)              :: warning
    REAL(stnd)                                    :: gtr, btr
    REAL(stnd), DIMENSION(kjpindex)               :: mean_dsg
    LOGICAL                                       :: OnceMore
    INTEGER(short), PARAMETER                     :: nitermax = 100
    INTEGER(short)                                :: niter
    INTEGER(short)                                :: nbad
    LOGICAL, DIMENSION(kjpindex,nvm)              :: lbad
    REAL(stnd)                                    :: gqseuil , eausup, wd1
    REAL(stnd), DIMENSION(nbdl+1)                 :: tmp_dl
    !
    !  0. we have only one flux field corresponding to water evaporated from the surface
    !
    DO jv=1,nvm
      DO ji = 1, kjpindex
        IF ( veget(ji,jv) .GT. zero ) THEN
          zeflux(ji,jv) = transpir(ji,jv)/veget(ji,jv)
          zpreci(ji,jv) = precisol(ji,jv)/veget(ji,jv)
        ELSE
          zeflux(ji,jv) = zero
          zpreci(ji,jv) = zero
        ENDIF
      ENDDO
    ENDDO
    !
    ! We need a test on the bare soil fraction because we can have bare soil evaporation even when 
    ! there is no bare soil because of transfers (snow for instance). This should only apply if there
    ! is vegetation but we do not test this case.
    ! 

    ! case 1 - there is vegetation and bare soil
    DO ji = 1, kjpindex
      IF ( (vegtot(ji) .GT. zero) .AND. (veget(ji,1) .GT. min_sechiba) ) THEN
        zeflux(ji,1) = vevapnu(ji)/veget(ji,1)
      ENDIF
    ENDDO

    ! case 2 - there is vegetation but no bare soil
    DO jv = 1, nvm
      DO ji = 1, kjpindex
        IF ( (vegtot(ji) .GT. zero) .AND. (veget(ji,1) .LE. min_sechiba)&
             & .AND. (veget(ji,jv) .GT. min_sechiba)) THEN
          zeflux(ji,jv) =  zeflux(ji,jv) + vevapnu(ji)/vegtot(ji)
        ENDIF
      ENDDO
    ENDDO

    !
    !  0.1 Other temporary variables
    !
    tmp_dl(1) = 0
    tmp_dl(2:nbdl+1) = diaglev(1:nbdl)
    !

    !
    ! 1.1 Transpiration for each vegetation type
    !     Evaporated water is taken out of the ground.
    !
    !
    DO jv=1,nvm
      DO ji=1,kjpindex 
        !
        gqsb(ji,jv) = gqsb(ji,jv) - zeflux(ji,jv)
        !
        ! 1.2 Add snow and ice melt, troughfall from vegetation and irrigation.
        !
        IF(vegtot(ji) .NE. zero) THEN
           ! We take care to add the irrigation only to the vegetated part if possible
           IF (ABS(vegtot(ji)-veget(ji,1)) .LE. min_sechiba) THEN
              gqsb(ji,jv) = gqsb(ji,jv) + zpreci(ji,jv) + (tot_melt(ji)+irrigation(ji))/vegtot(ji)
           ELSE
            IF(jv.le.1) THEN
              gqsb(ji,jv) = gqsb(ji,jv) + zpreci(ji,jv) + tot_melt(ji)/vegtot(ji)
             ELSE
              gqsb(ji,jv) = gqsb(ji,jv) + zpreci(ji,jv) + tot_melt(ji)/vegtot(ji)&
                         &+irrigation(ji)/(vegtot(ji)-veget(ji,1))
             ENDIF
           ENDIF
           !
           ! 1.3 We add the water returned from rivers to the lower reservoir.
           !
           bqsb(ji,jv) = bqsb(ji,jv) + returnflow(ji)/vegtot(ji)
           !
        ENDIF
        !
      END DO
    ENDDO
    ! 
    ! 1.3 Computes run-off
    !
    runoff(:,:) = zero
    ! 
    ! 1.4 Soil moisture is updated
    !
    warning(:,:) = .FALSE.
    DO jv=1,nvm
      DO ji = 1, kjpindex
        !
        runoff(ji,jv) = MAX(gqsb(ji,jv) + bqsb(ji,jv) - mx_eau_var(ji), zero)
        !
        IF (mx_eau_var(ji) .LE. (gqsb(ji,jv) + bqsb(ji,jv))) THEN
            !
            ! 1.4.1 Plus de reservoir de surface: le sol est sature
            !       d'eau. Le reservoir de surface est inexistant
            !       Tout est dans le reservoir de fond.
            !       Le ruissellement correspond a ce qui deborde.
            !
            gqsb(ji,jv) = zero
            dsg(ji,jv) = zero
            bqsb(ji,jv) = mx_eau_var(ji)
            dsp(ji,jv) = zero
            dss(ji,jv) = dsp (ji,jv)

        ELSEIF ((gqsb(ji,jv) + bqsb(ji,jv)).GE.zero) THEN
            !
            IF (gqsb(ji,jv) .GT. dsg(ji,jv) * ruu_ch(ji)) THEN
                !
                ! 1.4.2 On agrandit le reservoir de surface
                !       car il n y a pas eu ruissellement
                !       et toute l'eau du reservoir de surface
                !       tient dans un reservoir dont la taille
                !       est plus importante que l actuel. 
                !       La hauteur de sol sec dans le reservoir 
                !       de surface est alors nulle.
                !       Le reste ne bouge pas.
                !
                dsg(ji,jv) = gqsb(ji,jv) / ruu_ch(ji)
                dss(ji,jv) = zero
            ELSEIF (gqsb(ji,jv) .GT. zero ) THEN 
                !
                ! 1.4.3 L eau tient dans le reservoir de surface 
                !       tel qu il existe. 
                !       Calcul de la nouvelle hauteur de sol sec
                !       dans la couche de surface.
                !       Le reste ne bouge pas.
                !
                dss(ji,jv) = ((dsg(ji,jv) * ruu_ch(ji)) - gqsb(ji,jv)) / ruu_ch(ji) 
            ELSE
                !
                ! 1.4.4 La quantite d eau du reservoir de surface
                !       est negative. Cela revient a enlever
                !       cette quantite au reservoir profond.
                !       Le reservoir de surface est alors vide.
                !       (voir aussi 1.4.1)
                !
                bqsb(ji,jv) = bqsb(ji,jv) + gqsb(ji,jv)
                dsp(ji,jv) = dpu - bqsb(ji,jv) / ruu_ch(ji)
                gqsb(ji,jv) = zero
                dsg(ji,jv) = zero
                dss(ji,jv) = dsp(ji,jv)
            END IF

        ELSE 
            !
            ! 1.4.5 Le reservoir profond est aussi asseche.
            !       La quantite d eau a enlever depasse la quantite 
            !       disponible dans le reservoir profond.
            !
            !
            ! Ceci ne devrait jamais arriver plus d'une fois par point. C-a-d une fois la valeur negative
            ! atteinte les flux doivent etre nuls. On ne signal que ce cas donc.
            ! 
            IF ( ( zeflux(ji,jv) .GT. 0.0 ) .AND. &
                 ( gqsb(ji,jv) + bqsb(ji,jv) .LT. -1.e-10 ) ) THEN
               warning(ji,jv) = .TRUE.
               ! WRITE (numout,*) 'WARNING! Soil Moisture will be negative' 
               ! WRITE (numout,*) 'ji, jv = ', ji,jv
               ! WRITE (numout,*) 'mx_eau_var = ', mx_eau_var(ji)
               ! WRITE (numout,*) 'veget, resdist =', veget(ji,jv), resdist(ji,jv)
               ! WRITE (numout,*) 'bqsb = ', bqsb(ji,jv)
               ! WRITE (numout,*) 'gqsb = ', gqsb(ji,jv)
               ! WRITE (numout,*) 'dss = ', dss(ji,jv)
               ! WRITE (numout,*) 'dsg = ', dsg(ji,jv)
               ! WRITE (numout,*) 'dsp = ', dsp(ji,jv)
               ! WRITE (numout,*) 'humrel = ', humrel(ji, jv)
               ! WRITE (numout,*) 'Soil evaporation = ', zeflux(ji,jv)
               ! WRITE (numout,*) 'input = ',precisol(ji, jv), tot_melt(ji)
               ! WRITE (numout,*) '============================'
            ENDIF
            !
            bqsb(ji,jv) = gqsb(ji,jv) + bqsb(ji,jv)
            dsp(ji,jv) = dpu
            gqsb(ji,jv) = zero
            dsg(ji,jv) = zero
            dss(ji,jv) = dsp(ji,jv)
            !
        ENDIF
        !
      ENDDO
    ENDDO
    !
    nbad = COUNT( warning(:,:) .EQV. .TRUE. )
    IF ( nbad .GT. 0 ) THEN
      WRITE(numout,*) 'hydrol_soil: WARNING! Soil moisture was negative at', &
                       nbad, ' points:'
      DO jv = 1, nvm
        DO ji = 1, kjpindex
          IF ( warning(ji,jv) ) THEN
            WRITE(numout,*) '  ji,jv = ', ji,jv
            WRITE (numout,*) 'mx_eau_var = ', mx_eau_var(ji)
            WRITE (numout,*) 'veget, resdist =', veget(ji,jv), resdist(ji,jv)
            WRITE (numout,*) 'bqsb = ', bqsb(ji,jv)
            WRITE (numout,*) 'gqsb = ', gqsb(ji,jv)
            WRITE (numout,*) 'dss = ', dss(ji,jv)
            WRITE (numout,*) 'dsg = ', dsg(ji,jv)
            WRITE (numout,*) 'dsp = ', dsp(ji,jv)
            WRITE (numout,*) 'humrel = ', humrel(ji, jv)
            WRITE (numout,*) 'Soil evaporation = ', zeflux(ji,jv)
            WRITE (numout,*) 'input = ',precisol(ji, jv), tot_melt(ji)
            WRITE (numout,*) '============================'
          ENDIF
        ENDDO
      ENDDO
    ENDIF
    !
    !  2.0 Very large upper reservoirs or very close upper and lower reservoirs
    !         can be deadlock situations for Choisnel. They are handled here
    !
    IF (long_print) PRINT *, 'hydro_soil 2.0 : Resolve deadlocks'
    DO jv=1,nvm
      DO ji=1,kjpindex
        !
        !   2.1 the two reservoirs are very close to each other
        !
        IF ( ABS(dsp(ji,jv)-dsg(ji,jv)) .LT. min_resdis ) THEN
            bqsb(ji,jv) = bqsb(ji,jv) + gqsb(ji,jv)
            dsp(ji,jv) = dpu - bqsb(ji,jv) / ruu_ch(ji)
            gqsb(ji,jv) = zero
            dsg(ji,jv) = zero
            dss(ji,jv) = dsp(ji,jv)
        ENDIF
        !
        !  2.2 Draine some water from the upper to the lower reservoir
        !
        gqseuil = min_resdis * deux * ruu_ch(ji)
        eausup = dsg(ji,jv) * ruu_ch(ji)
        wd1 = .75*eausup
        !
        IF (eausup .GT. gqseuil) THEN
            gdrainage(ji,jv) = min_drain *  (gqsb(ji,jv)/eausup)
            !
            IF ( gqsb(ji,jv) .GE. wd1 .AND. dsg(ji,jv) .GT. 0.10 ) THEN
                !
                gdrainage(ji,jv) = gdrainage(ji,jv) + &
                   (max_drain-min_drain)*((gqsb(ji,jv)-wd1) / (eausup-wd1))**exp_drain
                !
            ENDIF
            !
            gdrainage(ji,jv)=MIN(gdrainage(ji,jv), MAX(gqsb(ji,jv), zero))
            !
        ELSE
            gdrainage(ji,jv)=zero
        ENDIF
        !
        gqsb(ji,jv) = gqsb(ji,jv) -  gdrainage(ji,jv)
        bqsb(ji,jv) = bqsb(ji,jv) + gdrainage(ji,jv)
        dsg(ji,jv) = dsg(ji,jv) -  gdrainage(ji,jv) / ruu_ch(ji)
        dsp(ji,jv) = dpu - bqsb(ji,jv)/ruu_ch(ji)
        !
        !
      ENDDO
      !
    ENDDO

    !
    !   3.0 Diffusion of water between the reservoirs of the different plants
    !
    IF (long_print) PRINT *, 'hydrol_soil 3.0 : Vertical diffusion'

    mean_bqsb(:) = 0.
    mean_gqsb(:) = 0.
    DO jv = 1, nvm
      DO ji = 1, kjpindex
        IF ( vegtot(ji) .GT. zero ) THEN
          mean_bqsb(ji) = mean_bqsb(ji) + veget(ji,jv)/vegtot(ji)*bqsb(ji,jv)
          mean_gqsb(ji) = mean_gqsb(ji) + veget(ji,jv)/vegtot(ji)*gqsb(ji,jv)
        ENDIF
      ENDDO
    ENDDO

    OnceMore = .TRUE.
    niter = 0

    ! nitermax prevents infinite loops (should actually never occur)
    DO WHILE ( OnceMore .AND. ( niter .LT. nitermax ) ) 
      !
      niter = niter + 1
      !
      ! where do we have to do something?
      lbad(:,:) = ( ( dsp(:,:) .LT. dsg(:,:) ) .AND. &
                    ( dsg(:,:) .GT. zero ) .AND. &
                    ( veget(:,:) .GT. zero ) )
      !
      ! if there are no such points any more, we'll do no more iteration
      IF ( COUNT( lbad(:,:) ) .EQ. 0 ) OnceMore = .FALSE.
      !
      DO jv = 1, nvm
        !
        DO ji = 1, kjpindex
          IF ( veget(ji,jv) .GT. 0. ) THEN
            !
            bqsb(ji,jv) = mean_bqsb(ji)
            dsp(ji,jv) = dpu - bqsb(ji,jv)/ruu_ch(ji)
          ENDIF
          !
        ENDDO
        !
        DO ji = 1, kjpindex
          IF ( lbad(ji,jv) ) THEN
            !
            runoff(ji,jv) = runoff(ji,jv) + &
                            MAX( bqsb(ji,jv) + gqsb(ji,jv) - mx_eau_var(ji), zero)
            !
            bqsb(ji,jv) = MIN( bqsb(ji,jv) + gqsb(ji,jv), mx_eau_var(ji))
            !
            gqsb(ji,jv) = zero
            dsp(ji,jv) = dpu - bqsb(ji,jv)/ruu_ch(ji)
            dss(ji,jv) = dsp(ji,jv)
            dsg(ji,jv) = zero
            !
          ENDIF
        ENDDO
        !
      ENDDO
      !
      mean_bqsb(:) = 0.
      mean_gqsb(:) = 0.
      DO jv = 1, nvm
        DO ji = 1, kjpindex
          IF ( vegtot(ji) .GT. zero ) THEN
            mean_bqsb(ji) = mean_bqsb(ji) + veget(ji,jv)/vegtot(ji)*bqsb(ji,jv)
            mean_gqsb(ji) = mean_gqsb(ji) + veget(ji,jv)/vegtot(ji)*gqsb(ji,jv)
          ENDIF
        ENDDO
      ENDDO
      !
    ENDDO

    !
    ! 4. computes total runoff
    !
    IF (long_print) PRINT *, 'hydro_soil 4.0: Computes total runoff'

    run_off_tot(:) = zero
    DO ji = 1, kjpindex
      IF ( vegtot(ji) .GT. zero ) THEN
       DO jv = 1, nvm
        run_off_tot(ji) = run_off_tot(ji) + (runoff(ji,jv)*veget(ji,jv))
       ENDDO
      ELSE
        run_off_tot(ji) = tot_melt(ji) + irrigation(ji)
      ENDIF
    ENDDO
    !
    ! 4.1 We estimate some drainage !
    !
    drainage(:) = 0.95 * run_off_tot(:)
    run_off_tot(:) = run_off_tot(:) - drainage(:)
    !
    ! 5. Some averaged diagnostics
    !
    IF (long_print) PRINT *, 'hydro_soil 5.0: Diagnostics'

    !
    ! 5.1 reset dsg if necessary
    !
    WHERE (gqsb(:,:) .LE. zero) dsg(:,:) = zero
    !
    DO ji=1,kjpindex
      !
      !  5.2 Compute an average moisture profile
      !
      mean_dsg(ji) = mean_gqsb(ji)/ruu_ch(ji)
      !
    ENDDO

    !
    ! 6. Compute the moisture stress on vegetation
    !
    IF (long_print) PRINT *, 'hydro_soil 6.0 : Moisture stress'

    DO jv = 1, nvm
      DO ji=1,kjpindex
         !
         ! computes relative surface humidity
         !
         ! Only use the standard formulas if total soil moisture is larger than zero.
         ! Else stress functions are set to zero. 
         ! This will avoid that large negative soil moisture accumulate over time by the
         ! the creation of small skin reservoirs which evaporate quickly.
         !
! Correction Gerhard, variable non initialisee dans certains cas
         a_subgrd(ji)=0.
         IF ( gqsb(ji,jv)+bqsb(ji,jv) .GT. zero ) THEN
            !
            IF (dsg(ji,jv).EQ. zero .OR. gqsb(ji,jv).EQ.zero) THEN
               humrel(ji,jv) = EXP( - humcste(jv) * dpu * (dsp(ji,jv)/dpu) )
               dsg(ji,jv) = zero
               !
               ! if the dry soil height is larger than the one corresponding
               ! to the wilting point, or negative lower soil moisture : humrel is 0.0
               !
               IF (dsp(ji,jv).GT.(dpu - (qwilt / ruu_ch(ji))) .OR. bqsb(ji,jv).LT.zero) THEN
                  humrel(ji,jv) = zero
               ENDIF
               !
               ! In this case we can take for vegetation growth the same values for humrel and vegstress
               !
               vegstress(ji,jv) = humrel(ji,jv)
               !
            ELSE
             dsg_min=0.001
             if (0.eq.1) then
               zhumrel_lo(ji) = EXP( - humcste(jv) * dpu * (dsp(ji,jv)/dpu) )
!              zhumrel_up(ji) = EXP( - humcste(jv) * dpu * (dss(ji,jv)/dsg(ji,jv)) )

               dsseff=max(dss(ji,jv),dsg_min-(dsg(ji,jv)-dss(ji,jv)))

               zhumrel_up(ji)=EXP(-humcste(jv)*dpu*(dsseff/max(dsg(ji,jv),dsg_min)) )
               humrel(ji,jv) = MAX(zhumrel_lo(ji),zhumrel_up(ji))
               !
               ! As we need a slower variable for vegetation growth the stress is computed
               ! differently than in humrel.
               !
               zhumrel_lo(ji) = EXP( - humcste(jv) * dsp(ji,jv))
               zhumrel_up(ji) = EXP( - humcste(jv) * dss(ji,jv))
               vegstress(ji,jv) = zhumrel_lo(ji) + zhumrel_up(ji) - EXP( - humcste(jv) * dsg(ji,jv) ) 
               !
             else
               zhumrel_lo(ji) = EXP( - humcste(jv) * dsp(ji,jv))
               zhumrel_up(ji) = EXP( - humcste(jv) * dss(ji,jv))
               a_subgrd(ji)=min(max(dsg(ji,jv)-dss(ji,jv),0.)/dsg_min,1.)
               humrel(ji,jv)=a_subgrd(ji)*zhumrel_up(ji)+(1.-a_subgrd(ji))*zhumrel_lo(ji)
               vegstress(ji,jv) = zhumrel_lo(ji) + zhumrel_up(ji) - EXP( - humcste(jv) * dsg(ji,jv) ) 
             endif
            ENDIF
            !
         ELSE
            !
            humrel(ji,jv) = zero
            vegstress(ji,jv) = zero
            !
         ENDIF
        !
      ENDDO

    !
    ! 7. Diagnostics which are needed to carry information to other modules
    !
    ! 7.1 store hdry for other modules
    !
     if (jv.eq.1) then
         hdry(:) = a_subgrd(:)*dss(:,1) + (1.-a_subgrd(:))*dsp(:,1)
     endif
    ENDDO
    !
    ! 7.2 Relative soil moisture
    !
    DO jd = 1,nbdl
      DO ji = 1, kjpindex
         IF ( tmp_dl(jd+1) .LT. mean_dsg(ji)) THEN
            shumdiag(ji,jd) = mean_gqsb(ji)/mx_eau_var(ji)
         ELSE
            IF ( tmp_dl(jd) .LT. mean_dsg(ji)) THEN
               gtr = (mean_dsg(ji)-tmp_dl(jd))/(tmp_dl(jd+1)-tmp_dl(jd))
               btr = 1 - gtr
               shumdiag(ji,jd) = gtr*mean_gqsb(ji)/mx_eau_var(ji) + &
                    & btr*mean_bqsb(ji)/mx_eau_var(ji)
            ELSE
               shumdiag(ji,jd) = mean_bqsb(ji)/mx_eau_var(ji)
            ENDIF
         ENDIF
         shumdiag(ji,jd) = MAX(MIN(shumdiag(ji,jd), un), zero)
      ENDDO
    ENDDO

    !
    ! 8. litter humidity.
    !

    litterhumdiag(:) = EXP( - hdry(:) / hcrit_litter )

    ! special case: it has just been raining a few drops. The upper soil
    ! reservoir is ridiculously small, but the dry soil height is zero.
    ! Don't take it into account.

    WHERE ( ( hdry(:) .LT. min_sechiba ) .AND. &
            ( mean_dsg(:) .GT. min_sechiba ) .AND. ( mean_dsg(:) .LT. 5.E-4 ) )
      litterhumdiag(:) = 0.0
    ENDWHERE

    IF (long_print) WRITE (numout,*) ' hydrol_soil done '

  END SUBROUTINE hydrol_soil
  !!
  !! This routines checks the water balance. First it gets the total
  !! amount of water and then it compares the increments with the fluxes.
  !! The computation is only done over the soil area as over glaciers (and lakes?)
  !! we do not have water conservation.
  !!
  !! This verification does not make much sense in REAL*4 as the precision is the same as some
  !! of the fluxes
  !!
  SUBROUTINE hydrol_waterbal (kjpindex, index, first_call, dtradia, veget, totfrac_nobio, qsintveg, snow, snow_nobio,&
       & precip_rain, precip_snow, returnflow, irrigation, tot_melt, vevapwet, transpir, vevapnu, vevapsno, run_off_tot, &
       & drainage)
    !
    !
    !
    INTEGER(long), INTENT (in)                        :: kjpindex     !! Domain size
    INTEGER(long),DIMENSION (kjpindex), INTENT (in)   :: index        !! Indeces of the points on the map
    LOGICAL, INTENT (in)                              :: first_call   !! At which time is this routine called ?
    REAL(stnd), INTENT (in)                           :: dtradia      !! Time step in seconds
    !
    REAL(stnd),DIMENSION (kjpindex,nvm), INTENT (in)  :: veget        !! Fraction of vegetation type 
    REAL(stnd),DIMENSION (kjpindex), INTENT (in)      :: totfrac_nobio!! Total fraction of continental ice+lakes+...
    REAL(stnd),DIMENSION (kjpindex,nvm), INTENT (in)  :: qsintveg     !! Water on vegetation due to interception
    REAL(stnd),DIMENSION (kjpindex), INTENT (in)      :: snow         !! Snow mass [Kg/m^2]
    REAL(stnd), DIMENSION (kjpindex,nnobio), INTENT(inout)  :: snow_nobio    !!Ice water balance
    !
    REAL(stnd),DIMENSION (kjpindex), INTENT (in)      :: precip_rain  !! Rain precipitation
    REAL(stnd),DIMENSION (kjpindex), INTENT (in)      :: precip_snow  !! Snow precipitation
    REAL(stnd),DIMENSION (kjpindex), INTENT (in)      :: returnflow   !! Water returning from routing to the deep reservoir
    REAL(stnd),DIMENSION (kjpindex), INTENT (in)      :: irrigation   !! Water from irrigation
    REAL(stnd),DIMENSION (kjpindex), INTENT (in)      :: tot_melt     !! Total melt
    !
    REAL(stnd),DIMENSION (kjpindex,nvm), INTENT (in)  :: vevapwet     !! Interception loss
    REAL(stnd),DIMENSION (kjpindex,nvm), INTENT (in)  :: transpir     !! Transpiration
    REAL(stnd),DIMENSION (kjpindex), INTENT (in)      :: vevapnu      !! Bare soil evaporation
    REAL(stnd),DIMENSION (kjpindex), INTENT (in)      :: vevapsno     !! Snow evaporation
    REAL(stnd),DIMENSION (kjpindex), INTENT (in)      :: run_off_tot  !! Total runoff
    REAL(stnd),DIMENSION (kjpindex), INTENT (in)      :: drainage     !! Drainage
    !
    !  LOCAL
    !
    INTEGER(long) :: ji
    REAL(stnd) :: watveg, delta_water, tot_flux, allowed_err
    !
    !
    !
    IF ( first_call ) THEN

       tot_water_beg(:) = zero

       DO ji = 1, kjpindex
          watveg = SUM(qsintveg(ji,:))
          tot_water_beg(ji) = (mean_bqsb(ji) + mean_gqsb(ji))*vegtot(ji) + watveg + &
            & snow(ji) + SUM(snow_nobio(ji,:))
       ENDDO
       tot_water_end(:) = tot_water_beg(:)

       RETURN

    ENDIF
    !
    !  Check the water balance
    !
    tot_water_end(:) = zero
    !
    DO ji = 1, kjpindex
       !
       ! If the fraction of ice, lakes, etc. does not complement the vegetation fraction then we do not
       ! need to go any further
       !
       IF ( (un - (totfrac_nobio(ji) + vegtot(ji))) .GT. EPSILON(un) ) THEN
          WRITE(*,*) 'HYDROL problem in vegetation or frac_nobio on point ', ji
          WRITE(*,*) 'totfrac_nobio : ', totfrac_nobio(ji)
          WRITE(*,*) 'vegetation fraction : ', vegtot(ji)
          STOP 'in hydrol_waterbal'
       ENDIF
       !
       watveg = SUM(qsintveg(ji,:))
          tot_water_end(ji) = (mean_bqsb(ji) + mean_gqsb(ji))*vegtot(ji) + watveg + &
            & snow(ji) + SUM(snow_nobio(ji,:))
       !
       delta_water = tot_water_end(ji) - tot_water_beg(ji)
       !
        tot_flux =  precip_rain(ji) + precip_snow(ji) + returnflow(ji) + irrigation(ji) - &
             & SUM(vevapwet(ji,:)) - SUM(transpir(ji,:)) - vevapnu(ji) - vevapsno(ji) - &
             & run_off_tot(ji) - drainage(ji) 
       !
       !  Set some precision ! This is a wild guess and corresponds to what works on an IEEE machine
       !  under double precision (REAL*8).
       !
       allowed_err = 50000*EPSILON(un) 
       !
       IF ( ABS(delta_water-tot_flux) .GT. allowed_err ) THEN
          WRITE(*,*) 'HYDROL does not conserve water. The erroneous point is : ', ji
          WRITE(*,*) 'The error in mm/d is :', (delta_water-tot_flux)/dtradia, ' and in mm/dt : ', delta_water-tot_flux
          WRITE(*,*) 'delta_water : ', delta_water, ' tot_flux : ', tot_flux
          WRITE(*,*) 'Actual and allowed error : ', ABS(delta_water-tot_flux), allowed_err
          WRITE(*,*) 'vegtot : ', vegtot(ji)
          WRITE(*,*) 'precip_rain : ', precip_rain(ji)
          WRITE(*,*) 'precip_snow : ',  precip_snow(ji)
          WRITE(*,*) 'Water from irrigation : ', returnflow(ji), irrigation(ji)
          WRITE(*,*) 'Total water in soil :', mean_bqsb(ji) + mean_gqsb(ji)
          WRITE(*,*) 'Water on vegetation :', watveg
          WRITE(*,*) 'Snow mass :', snow(ji)
          WRITE(*,*) 'Snow mass on ice :', SUM(snow_nobio(ji,:))
          WRITE(*,*) 'Melt water :', tot_melt(ji)
          WRITE(*,*) 'evapwet : ', vevapwet(ji,:)
          WRITE(*,*) 'transpir : ', transpir(ji,:)
          WRITE(*,*) 'evapnu, evapsno : ', vevapnu(ji), vevapsno(ji)

          WRITE(*,*) 'drainage : ', drainage(ji)
          STOP 'in hydrol_waterbal'
       ENDIF
       !
    ENDDO
    !
    ! Transfer the total water amount at the end of the current timestep top the begining of the next one.
    !
    tot_water_beg = tot_water_end
    !
  END SUBROUTINE hydrol_waterbal
  !
  !  This routine computes the changes in soil moisture and interception storage for the ALMA outputs
  !
  SUBROUTINE hydrol_alma (kjpindex, index, first_call, qsintveg, snow, snow_nobio, soilwet)
    !
    INTEGER(long), INTENT (in)                        :: kjpindex     !! Domain size
    INTEGER(long),DIMENSION (kjpindex), INTENT (in)   :: index        !! Indeces of the points on the map
    LOGICAL, INTENT (in)                              :: first_call   !! At which time is this routine called ?
    !
    REAL(stnd),DIMENSION (kjpindex,nvm), INTENT (in)  :: qsintveg     !! Water on vegetation due to interception
    REAL(stnd),DIMENSION (kjpindex), INTENT (in)      :: snow         !! Snow water equivalent
    REAL(stnd),DIMENSION (kjpindex), INTENT (out)      :: soilwet     !! Soil wetness
    REAL(stnd),DIMENSION (kjpindex,nnobio), INTENT (in) :: snow_nobio     !! Water balance on ice, lakes, .. [Kg/m^2]
    !
    !  LOCAL
    !
    INTEGER(long) :: ji
    REAL(stnd) :: watveg 
    !
    !
    !
    IF ( first_call ) THEN

       tot_watveg_beg(:) = zero
       tot_watsoil_beg(:) = zero
       snow_beg(:)        = zero 
       !
       DO ji = 1, kjpindex
          watveg = SUM(qsintveg(ji,:))
          tot_watveg_beg(ji) = watveg
          tot_watsoil_beg(ji) = mean_bqsb(ji) + mean_gqsb(ji)
          snow_beg(ji)        = snow(ji)+ SUM(snow_nobio(ji,:))  
       ENDDO
       !
       tot_watveg_end(:) = tot_watveg_beg(:)
       tot_watsoil_end(:) = tot_watsoil_beg(:)
       snow_end(:)        = snow_beg(:)

       RETURN

    ENDIF
    !
    ! Calculate the values for the end of the time step 
    !
    tot_watveg_end(:) = zero
    tot_watsoil_end(:) = zero
    snow_end(:) = zero
    delintercept(:) = zero
    delsoilmoist(:) = zero
    delswe(:) = zero
    !
    DO ji = 1, kjpindex
       watveg = SUM(qsintveg(ji,:))
       tot_watveg_end(ji) = watveg
       tot_watsoil_end(ji) = mean_bqsb(ji) + mean_gqsb(ji)
       snow_end(ji) = snow(ji)+ SUM(snow_nobio(ji,:))
       !  
       delintercept(ji) = tot_watveg_end(ji) - tot_watveg_beg(ji)
       delsoilmoist(ji) = tot_watsoil_end(ji) - tot_watsoil_beg(ji)
       delswe(ji)       = snow_end(ji) - snow_beg(ji)
       !
       !
    ENDDO
    !
    !
    ! Transfer the total water amount at the end of the current timestep top the begining of the next one.
    !
    tot_watveg_beg = tot_watveg_end
    tot_watsoil_beg = tot_watsoil_end
    snow_beg(:) = snow_end(:)
    !
    DO ji = 1,kjpindex
       soilwet(ji) = tot_watsoil_end(ji) / mx_eau_var(ji)
    ENDDO
    !
  END SUBROUTINE hydrol_alma
  !-
  SUBROUTINE hydrol_hdiff(kjpindex, dtradia, veget, ruu_ch, gqsb, bqsb, dsg, dss, dsp)

    INTEGER(long), INTENT (in)                          :: kjpindex     !! Domain size
    REAL(stnd), INTENT (in)                             :: dtradia      !! time step (s)
    REAL(stnd),DIMENSION (kjpindex,nvm), INTENT (in)    :: veget        !! Fraction of vegetation type
    REAL(stnd),DIMENSION (kjpindex), INTENT (in)        :: ruu_ch       !! Quantite d'eau maximum

    REAL(stnd), DIMENSION (kjpindex,nvm), INTENT(inout) :: gqsb         !! Hauteur d'eau dans le reservoir de surface
    REAL(stnd), DIMENSION (kjpindex,nvm), INTENT(inout) :: bqsb         !! Hauteur d'eau dans le reservoir profond
    REAL(stnd), DIMENSION (kjpindex,nvm), INTENT(inout) :: dsg          !! Hauteur du reservoir de surface
    REAL(stnd), DIMENSION (kjpindex,nvm), INTENT(inout) :: dss          !! Hauteur au dessus du reservoir de surface
    REAL(stnd), DIMENSION (kjpindex,nvm), INTENT(inout) :: dsp          !! Hauteur au dessus du reservoir profond

    REAL(stnd), DIMENSION (kjpindex)                    :: bqsb_mean
    REAL(stnd), DIMENSION (kjpindex)                    :: gqsb_mean
    REAL(stnd), DIMENSION (kjpindex)                    :: dss_mean
    REAL(stnd), DIMENSION (kjpindex)                    :: vegtot
    REAL(stnd)                                          :: x
    INTEGER(short)                                      :: ji,jv
    REAL(stnd), SAVE                                    :: tau_hdiff
    LOGICAL, SAVE                                       :: firstcall=.TRUE.

    IF ( firstcall ) THEN

      !Config  Key  = HYDROL_TAU_HDIFF
      !Config  Desc = time scale (s) for horizontal diffusion of water
      !Config  Def  = 86400.
      !Config  If   = HYDROL_OK_HDIFF
      !Config  Help = Defines how fast diffusion occurs horizontally between
      !Config         the individual PFTs' water reservoirs. If infinite, no
      !Config         diffusion.

      tau_hdiff = 86400.
      CALL getin('HYDROL_TAU_HDIFF',tau_hdiff)

      WRITE (numout,*) 'Hydrol: Horizontal diffusion, tau (s)=',tau_hdiff

      firstcall = .FALSE.

    ENDIF

    ! Calculate mean values
    ! could be done with SUM instruction but this kills vectorization
    !
    bqsb_mean(:) = 0.0
    gqsb_mean(:) = 0.0
    dss_mean(:) = 0.0
    vegtot(:) = 0.0
    !
    DO jv = 1, nvm
      DO ji = 1, kjpindex
        bqsb_mean(ji) = bqsb_mean(ji) + veget(ji,jv)*bqsb(ji,jv)
        gqsb_mean(ji) = gqsb_mean(ji) + veget(ji,jv)*gqsb(ji,jv)
        dss_mean(ji) = dss_mean(ji) + veget(ji,jv)*dss(ji,jv)
        vegtot(ji) = vegtot(ji) + veget(ji,jv)
      ENDDO
    ENDDO
 
    DO ji = 1, kjpindex
      IF (vegtot(ji) .GT. zero) THEN
        bqsb_mean(ji) = bqsb_mean(ji)/vegtot(ji)
        gqsb_mean(ji) = gqsb_mean(ji)/vegtot(ji)
        dss_mean(ji) = dss_mean(ji)/vegtot(ji)
      ENDIF
    ENDDO

    ! relax values towards mean.
    !
    x = MAX( zero, MIN( dtradia/tau_hdiff, un ) )
    !
    DO jv = 1, nvm
      DO ji = 1, kjpindex
        !
        bqsb(ji,jv) = (un-x) * bqsb(ji,jv) + x * bqsb_mean(ji)
        gqsb(ji,jv) = (un-x) * gqsb(ji,jv) + x * gqsb_mean(ji)
        dss(ji,jv) = (un-x) * dss(ji,jv) + x * dss_mean(ji)
        !
        IF (gqsb(ji,jv) .LT. min_sechiba) THEN
          dsg(ji,jv) = 0.0
        ELSE
          dsg(ji,jv) = (dss(ji,jv) * ruu_ch(ji) + gqsb(ji,jv)) / ruu_ch(ji)
        ENDIF
        dsp(ji,jv) = dpu - bqsb(ji,jv) / ruu_ch(ji)
        !
      ENDDO
    ENDDO

  END SUBROUTINE hydrol_hdiff
  !
END MODULE hydrol
